#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Настройки
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################

import warnings

# Подавление Warning
for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

import os  # Взаимодействие с файловой системой
import re  # Регулярные выражения

from typing import List

# Персональные
from oceanai.modules.core.messages import Messages  # Сообщения

# ######################################################################################################################
# Константы
# ######################################################################################################################

COLOR_INFO: str = "#1776D2"  # Цвет текста содержащего информацию (шестнадцатеричный код)
COLOR_SIMPLE: str = "#666"  # Цвет обычного текста (шестнадцатеричный код)
COLOR_ERR: str = "#FF0000"  # Цвет текста содержащего ошибку (шестнадцатеричный код)
COLOR_TRUE: str = "#008001"  # Цвет текста содержащего положительную информацию (шестнадцатеричный код)
BOLD_TEXT: bool = True  # Жирное начертание текста
CHUNK_SIZE: int = 1000000  # Размер загрузки файла из сети за 1 шаг
EXT: List[str] = []  # Расширения искомых файлов
IGNORE_DIRS: List[str] = []  # Директории не входящие в выборку
# Названия ключей для DataFrame набора данных
KEYS_DATASET: List[str] = ["Path", "Openness", "Conscientiousness", "Extraversion", "Agreeableness", "Non-Neuroticism"]
NUM_TO_DF_DISPLAY: int = 30  # Количество строк для отображения в таблицах
PATH_TO_DATASET: str = ""  # Директория набора данных
PATH_TO_SAVE: str = "./models"  # Директория для сохранения данных
PATH_TO_LOGS: str = "./logs"  # Директория для сохранения LOG файлов
TEXT_RUNTIME: str = ""  # Текст времени выполнения


# ######################################################################################################################
# Настройки
# ######################################################################################################################
@dataclass
class Settings(Messages):
    """Класс для настроек

    Args:
        lang (str): Смотреть :attr:`~oceanai.modules.core.language.Language.lang`
        color_simple (str): Цвет обычного текста (шестнадцатеричный код)
        color_info (str): Цвет текста содержащего информацию (шестнадцатеричный код)
        color_err (str): Цвет текста содержащего ошибку (шестнадцатеричный код)
        color_true (str): Цвет текста содержащего положительную информацию (шестнадцатеричный код)
        bold_text (bool): Жирное начертание текста
        num_to_df_display (int): Количество строк для отображения в таблицах
        text_runtime (str): Текст времени выполнения
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    # Цвет текстов
    color_simple: str = COLOR_SIMPLE
    """
    str: Цвет обычного текста (шестнадцатеричный код)

    .. dropdown:: Примеры

        :bdg-success:`Верно` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 1
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings()
            print(settings.color_simple, settings.color_simple_)

        .. output-cell::
            :execution-count: 1
            :linenos:

            #666 #666

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 2
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_simple = '#666')
            print(settings.color_simple, settings.color_simple_)

        .. output-cell::
            :execution-count: 2
            :linenos:

            #666 #666

        :bdg-light:`-- 3 --`

        .. code-cell:: python
            :execution-count: 3
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_simple = '#222')
            print(settings.color_simple, settings.color_simple_)

        .. output-cell::
            :execution-count: 3
            :linenos:

            #222 #222

        :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 4
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_simple = 1)
            print(settings.color_simple, settings.color_simple_)

        .. output-cell::
            :execution-count: 4
            :linenos:

            #666 #666

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 5
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_simple = {1, 2, 3})
            print(settings.color_simple, settings.color_simple_)

        .. output-cell::
            :execution-count: 5
            :linenos:

            #666 #666
    """

    color_info: str = COLOR_INFO
    """
    str: Цвет текста содержащего информацию (шестнадцатеричный код)

    .. dropdown:: Примеры

        :bdg-success:`Верно` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 1
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings()
            print(settings.color_info, settings.color_info_)

        .. output-cell::
            :execution-count: 1
            :linenos:

            #1776D2 #1776D2

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 2
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_info = '#1776D2')
            print(settings.color_info, settings.color_info_)

        .. output-cell::
            :execution-count: 2
            :linenos:

            #1776D2 #1776D2

        :bdg-light:`-- 3 --`

        .. code-cell:: python
            :execution-count: 3
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_info = '#42F')
            print(settings.color_info, settings.color_info_)

        .. output-cell::
            :execution-count: 3
            :linenos:

            #42F #42F

        :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 4
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_info = 1)
            print(settings.color_info, settings.color_info_)

        .. output-cell::
            :execution-count: 4
            :linenos:

            #1776D2 #1776D2

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 5
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_info = [])
            print(settings.color_info, settings.color_info_)

        .. output-cell::
            :execution-count: 5
            :linenos:

            #1776D2 #1776D2
    """

    color_err: str = COLOR_ERR
    """
    str: Цвет текста содержащего ошибку (шестнадцатеричный код)

    .. dropdown:: Примеры

        :bdg-success:`Верно` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 1
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings()
            print(settings.color_err, settings.color_err_)

        .. output-cell::
            :execution-count: 1
            :linenos:

            #FF0000 #FF0000

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 2
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_err = 'FF0000')
            print(settings.color_err, settings.color_err_)

        .. output-cell::
            :execution-count: 2
            :linenos:

            #FF0000 #FF0000

        :bdg-light:`-- 3 --`

        .. code-cell:: python
            :execution-count: 3
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_err = '#FF0')
            print(settings.color_err, settings.color_err_)

        .. output-cell::
            :execution-count: 3
            :linenos:

            #FF0 #FF0

        :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 4
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_err = 1)
            print(settings.color_err, settings.color_err_)

        .. output-cell::
            :execution-count: 4
            :linenos:

            #FF0000 #FF0000

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 5
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_err = [])
            print(settings.color_err, settings.color_err_)

        .. output-cell::
            :execution-count: 5
            :linenos:

            #FF0000 #FF0000
    """

    color_true: str = COLOR_TRUE
    """
    str: Цвет текста содержащего положительную информацию (шестнадцатеричный код)

    .. dropdown:: Примеры

        :bdg-success:`Верно` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 1
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings()
            print(settings.color_true, settings.color_true_)

        .. output-cell::
            :execution-count: 1
            :linenos:

            #008001 #008001

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 2
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_true = '#003332')
            print(settings.color_true, settings.color_true_)

        .. output-cell::
            :execution-count: 2
            :linenos:

            #003332 #003332

        :bdg-light:`-- 3 --`

        .. code-cell:: python
            :execution-count: 3
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_true = '#42F')
            print(settings.color_true, settings.color_true_)

        .. output-cell::
            :execution-count: 3
            :linenos:

            #42F #42F

        :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 4
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_true = 1)
            print(settings.color_true, settings.color_true_)

        .. output-cell::
            :execution-count: 4
            :linenos:

            #008001 #008001

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 5
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(color_true = [])
            print(settings.color_true, settings.color_true_)

        .. output-cell::
            :execution-count: 5
            :linenos:

            #008001 #008001
    """

    bold_text: bool = BOLD_TEXT
    """
    bool: Жирное начертание текста

    .. dropdown:: Примеры

        :bdg-success:`Верно` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 1
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(lang = 'ru')
            print(settings.bold_text, settings.bold_text_)

        .. output-cell::
            :execution-count: 1
            :linenos:

            True True

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 2
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(lang = 'ru', bold_text = True)
            print(settings.bold_text, settings.bold_text_)

        .. output-cell::
            :execution-count: 2
            :linenos:

            True True

        :bdg-light:`-- 3 --`

        .. code-cell:: python
            :execution-count: 3
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(lang = 'ru', bold_text = False)
            print(settings.bold_text, settings.bold_text_)

        .. output-cell::
            :execution-count: 3
            :linenos:

            False False

        :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 4
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(lang = 'ru', bold_text = 1)
            print(settings.bold_text, settings.bold_text_)

        .. output-cell::
            :execution-count: 4
            :linenos:

            True True

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 5
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(lang = 'ru', bold_text = 'какой-то_текст')
            print(settings.bold_text, settings.bold_text_)

        .. output-cell::
            :execution-count: 5
            :linenos:

            True True
    """

    text_runtime: str = TEXT_RUNTIME
    """
    str: Текст времени выполнения

    .. dropdown:: Примеры

        :bdg-success:`Верно` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 1
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings()
            print(settings.text_runtime, settings.text_runtime_)

        .. output-cell::
            :execution-count: 1
            :linenos:

            Время выполнения Время выполнения

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 2
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(text_runtime = 'Код выполнился за')
            print(settings.text_runtime, settings.text_runtime_)

        .. output-cell::
            :execution-count: 2
            :linenos:

            Код выполнился за Код выполнился за

        :bdg-light:`-- 3 --`

        .. code-cell:: python
            :execution-count: 3
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(text_runtime = 'Время выполнения')
            print(settings.text_runtime, settings.text_runtime_)

        .. output-cell::
            :execution-count: 3
            :linenos:

            Время выполнения Время выполнения

        :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 4
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(text_runtime = 1)
            print(settings.text_runtime, settings.text_runtime_)

        .. output-cell::
            :execution-count: 4
            :linenos:

            Время выполнения Время выполнения

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 5
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(text_runtime = {1, 2, 3})
            print(settings.text_runtime, settings.text_runtime_)

        .. output-cell::
            :execution-count: 5
            :linenos:

            Время выполнения Время выполнения
    """

    num_to_df_display: int = NUM_TO_DF_DISPLAY
    """
    int: Количество строк для отображения в таблицах

    .. dropdown:: Примеры

        :bdg-success:`Верно` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 1
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings()
            print(settings.num_to_df_display, settings.num_to_df_display_)

        .. output-cell::
            :execution-count: 1
            :linenos:

            30 30

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 2
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(num_to_df_display = 30)
            print(settings.num_to_df_display, settings.num_to_df_display_)

        .. output-cell::
            :execution-count: 2
            :linenos:

            30 30

        :bdg-light:`-- 3 --`

        .. code-cell:: python
            :execution-count: 3
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(num_to_df_display = 50)
            print(settings.num_to_df_display, settings.num_to_df_display_)

        .. output-cell::
            :execution-count: 3
            :linenos:

            50 50

        :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

        .. code-cell:: python
            :execution-count: 4
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(num_to_df_display = 0)
            print(settings.num_to_df_display, settings.num_to_df_display_)

        .. output-cell::
            :execution-count: 4
            :linenos:

            30 30

        :bdg-light:`-- 2 --`

        .. code-cell:: python
            :execution-count: 5
            :linenos:

            from oceanai.modules.core.settings import Settings

            settings = Settings(num_to_df_display = 'какой-то_текст')
            print(settings.num_to_df_display, settings.num_to_df_display_)

        .. output-cell::
            :execution-count: 5
            :linenos:

            30 30
    """

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self.__re_search_color: str = r"^#(?:[0-9a-fA-F]{3}){1,2}$"  # Регулярное выражение для корректности ввода цвета

        # Цвет текстов
        self.__color_simple_true: int = 0  # Счетчик изменения текста
        self.color_simple_: str = self.color_simple  # Обычный текст

        self.__color_info_true: int = 0  # Счетчик изменения текста
        self.color_info_: str = self.color_info  # Цвет текста содержащего информацию

        self.__color_true_true: int = 0  # Счетчик изменения текста
        self.color_true_: str = self.color_true  # Цвет текста содержащего положительную информацию

        self.__color_err_true: int = 0  # Счетчик изменения текста
        self.color_err_: str = self.color_err  # Цвет текста содержащего ошибку

        self.__bold_text_true: int = 0  # Счетчик изменения начертания текста
        self.bold_text_: bool = self.bold_text  # Жирное начертание текста

        self.__text_runtime_true: int = 0  # Счетчик изменения текста
        self.text_runtime_: str = self.text_runtime  # Текст времени выполнения

        self.__num_to_df_display_true: int = 0  # Счетчик изменения количества строк для отображения в таблицах
        self.num_to_df_display_: int = self.num_to_df_display  # Количество строк для отображения в таблицах

        self.chunk_size_: int = CHUNK_SIZE  # Размер загрузки файла из сети за 1 шаг

        self.path_to_save_: str = PATH_TO_SAVE  # Директория для сохранения данных
        self.path_to_dataset_: str = PATH_TO_DATASET  # Директория набора данных
        self.path_to_logs_: str = PATH_TO_LOGS  # Директория для сохранения LOG файлов

        self.ext_: List[str] = EXT  # Расширения искомых файлов

        self.ignore_dirs_: List[str] = IGNORE_DIRS  # Директории не входящие в выборку

        self.keys_dataset_: List[str] = KEYS_DATASET  # Названия ключей для DataFrame набора данных

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    @property
    def color_simple_(self) -> str:
        """Получение/установка цвета обычного текста

        Args:
            (str): Шестнадцатеричный код

        Returns:
            str: Шестнадцатеричный код

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(color_simple = '#111')
                print(settings.color_simple_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                #111

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_simple_ = '#444'
                print(settings.color_simple_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                #444

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_simple_ = 1
                print(settings.color_simple_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                #666

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_simple_ = ()
                print(settings.color_simple_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                #666
        """

        return self.color_simple

    @color_simple_.setter
    def color_simple_(self, color: str) -> None:
        """Установка цвета обычного текста"""

        try:
            # Проверка аргументов
            match = re.search(self.__re_search_color, color)
            if not match:
                raise TypeError
        except TypeError:
            if self.__color_simple_true == 0:
                self.color_simple = COLOR_SIMPLE
        else:
            self.color_simple = color
            self.__color_simple_true += 1  # Увеличение счетчика изменения цвета текста

    @property
    def color_info_(self) -> str:
        """Получение/установка цвета текста содержащего информацию

        Args:
            (str): Шестнадцатеричный код

        Returns:
            str: Шестнадцатеричный код

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(color_info = '#1776D2')
                print(settings.color_info_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                #1776D2

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_info_ = '#42F'
                print(settings.color_info_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                #42F

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_info_ = 1
                print(settings.color_info_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                #1776D2

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_info_ = ()
                print(settings.color_info_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                #1776D2
        """

        return self.color_info

    @color_info_.setter
    def color_info_(self, color: str) -> None:
        """Установка цвета текста содержащего информацию"""

        try:
            # Проверка аргументов
            match = re.search(self.__re_search_color, color)
            if not match:
                raise TypeError
        except TypeError:
            if self.__color_info_true == 0:
                self.color_info = COLOR_INFO
        else:
            self.color_info = color
            self.__color_info_true += 1  # Увеличение счетчика изменения цвета текста

    @property
    def color_true_(self) -> str:
        """Получение/установка цвета текста содержащего положительную информацию

        Args:
            (str): Шестнадцатеричный код

        Returns:
            str: Шестнадцатеричный код

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(color_true = '#008001')
                print(settings.color_true_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                #008001

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_true_ = '#42F'
                print(settings.color_true_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                #42F

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_true = 1
                print(settings.color_true)

            .. output-cell::
                :execution-count: 3
                :linenos:

                #008001

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_true_ = ()
                print(settings.color_true_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                #008001
        """

        return self.color_true

    @color_true_.setter
    def color_true_(self, color: str) -> None:
        """Установка цвета текста содержащего положительную информацию"""

        try:
            # Проверка аргументов
            match = re.search(self.__re_search_color, color)
            if not match:
                raise TypeError
        except TypeError:
            if self.__color_true_true == 0:
                self.color_true = COLOR_TRUE
        else:
            self.color_true = color
            self.__color_true_true += 1  # Увеличение счетчика изменения цвета текста

    @property
    def color_err_(self) -> str:
        """Получение/установка цвета текста содержащего ошибку

        Args:
            (str): Шестнадцатеричный код

        Returns:
            str: Шестнадцатеричный код

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(color_err = '#C22931')
                print(settings.color_err_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                #C22931

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_err_ = '#FF0'
                print(settings.color_err_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                #FF0

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_err_ = 1
                print(settings.color_err_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                #FF0000

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.color_err_ = {}
                print(settings.color_err_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                #FF0000
        """

        return self.color_err

    @color_err_.setter
    def color_err_(self, color: str) -> None:
        """Установка цвета текста содержащего ошибку"""

        try:
            # Проверка аргументов
            match = re.search(self.__re_search_color, color)
            if not match:
                raise TypeError
        except TypeError:
            if self.__color_err_true == 0:
                self.color_err = COLOR_ERR
        else:
            self.color_err = color
            self.__color_err_true += 1  # Увеличение счетчика изменения цвета текста

    @property
    def bold_text_(self) -> bool:
        """Получение/установка жирного начертания текста

        Args:
            (bool): **True** или **False**

        Returns:
            bool: **True** или **False**

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(lang = 'ru', bold_text = True)
                print(settings.bold_text_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                True

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(lang = 'ru', bold_text = True)
                settings.bold_text_ = False
                print(settings.bold_text_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                False

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(lang = 'en', bold_text = False)
                settings.bold_text_ = 1
                print(settings.bold_text_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                False

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(lang = 'ru', bold_text = True)
                settings.bold_text_ = 'какой-то_текст'
                print(settings.bold_text_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                True
        """

        return self.bold_text

    @bold_text_.setter
    def bold_text_(self, bold: bool) -> None:
        """Установка жирного начертания текста"""

        try:
            # Проверка аргументов
            if type(bold) is not bool:
                raise TypeError
        except TypeError:
            if self.__bold_text_true == 0:
                self.bold_text = BOLD_TEXT
        else:
            self.bold_text = bold
            self.__bold_text_true += 1  # Увеличение счетчика изменения начертания текста

    @property
    def text_runtime_(self) -> str:
        """Получение/установка текста времени выполнения

        Args:
            (str): Текст

        Returns:
            str: Текст

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(text_runtime = 'Время выполнения')
                print(settings.text_runtime_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                Время выполнения

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.text_runtime_ = 'Код выполнился за'
                print(settings.text_runtime_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                Код выполнился за

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.text_runtime_ = 1
                print(settings.text_runtime_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                Время выполнения

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.text_runtime_ = ()
                print(settings.text_runtime_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                Время выполнения
        """

        return self.text_runtime

    @text_runtime_.setter
    def text_runtime_(self, text: str) -> None:
        """Установка текста времени выполнения"""

        try:
            # Проверка аргументов
            if type(text) is not str or len(text) < 1:
                raise TypeError
        except TypeError:
            if self.__text_runtime_true == 0:
                self.text_runtime = self._text_runtime
        else:
            self.text_runtime = text
            self.__text_runtime_true += 1  # Увеличение счетчика изменения текста времени выполнения

    @property
    def num_to_df_display_(self) -> int:
        """Получение/установка количества строк для отображения в таблицах

        Args:
            (int): Количество строк

        Returns:
            int: Количество строк

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings(num_to_df_display = 30)
                print(settings.num_to_df_display_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                30

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.num_to_df_display_ = 50
                print(settings.num_to_df_display_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                50

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.num_to_df_display_ = 0
                print(settings.num_to_df_display_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                30

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.num_to_df_display_ = ()
                print(settings.num_to_df_display_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                30
        """

        return self.num_to_df_display

    # Установка количества строк для отображения в таблицах
    @num_to_df_display_.setter
    def num_to_df_display_(self, num: int) -> None:
        """Установка количества строк для отображения в таблицах"""

        try:
            # Проверка аргументов
            if type(num) is not int or num < 1 or num > 50:
                raise TypeError
        except TypeError:
            if self.__num_to_df_display_true == 0:
                self.num_to_df_display = NUM_TO_DF_DISPLAY
        else:
            self.num_to_df_display = num
            # Увеличение счетчика изменения количества строк для отображения в таблицах
            self.__num_to_df_display_true += 1

    @property
    def path_to_save_(self) -> str:
        """Получение/установка директории для сохранения данных

        Args:
            (str): Директория для сохранения данных

        Returns:
            str: Директория для сохранения данных

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                print(settings.path_to_save_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                models

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_save_ = './models/Audio'
                print(settings.path_to_save_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                models/Audio

            :bdg-light:`-- 3 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_save_ = ''
                print(settings.path_to_save_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                .

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_save_ = [2, []]
                print(settings.path_to_save_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                models

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 5
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_save_ = {'a': 1, 'b': 2}
                print(settings.path_to_save_)

            .. output-cell::
                :execution-count: 5
                :linenos:

                models
        """

        return self._path_to_save

    @path_to_save_.setter
    def path_to_save_(self, path: str) -> None:
        """Установка директории для сохранения данных"""

        if type(path) is str:
            self._path_to_save = os.path.normpath(path)

    @property
    def path_to_logs_(self) -> str:
        """Получение/установка директории для сохранения LOG файлов

        Args:
            (str): Директория для сохранения LOG файлов

        Returns:
            str: Директория для сохранения LOG файлов

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                print(settings.path_to_logs_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                logs

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_logs_ = './logs/DF'
                print(settings.path_to_logs_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                logs/DF

            :bdg-light:`-- 3 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_logs_ = ''
                print(settings.path_to_logs_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                .

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_logs_ = [2, []]
                print(settings.path_to_logs_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                logs

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 5
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_logs_ = {'a': 1, 'b': 2}
                print(settings.path_to_logs_)

            .. output-cell::
                :execution-count: 5
                :linenos:

                logs
        """

        return self._path_to_logs

    @path_to_logs_.setter
    def path_to_logs_(self, path: str) -> None:
        """Установка директории для сохранения LOG файлов"""

        if type(path) is str:
            self._path_to_logs = os.path.normpath(path)

    @property
    def chunk_size_(self) -> int:
        """Получение/установка размера загрузки файла из сети за 1 шаг

        Args:
            (int): Размер загрузки файла из сети за 1 шаг

        Returns:
            int: Размер загрузки файла из сети за 1 шаг

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                print(settings.chunk_size_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                1000000

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.chunk_size_ = 2000000
                print(settings.chunk_size_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                2000000

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.chunk_size_ = -1
                print(settings.chunk_size_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                1000000

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.chunk_size_ = False
                print(settings.chunk_size_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                1000000

            :bdg-light:`-- 3 --`

            .. code-cell:: python
                :execution-count: 5
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.chunk_size_ = 'какой-то_текст'
                print(settings.chunk_size_)

            .. output-cell::
                :execution-count: 5
                :linenos:

                1000000
        """

        return self._chunk_size

    @chunk_size_.setter
    def chunk_size_(self, size: int) -> None:
        """Установка директории для сохранения данных"""

        if type(size) is int and size > 0:
            self._chunk_size = size

    @property
    def path_to_dataset_(self) -> str:
        """Получение/установка директории набора данных

        Args:
            (str): Директория набора данных

        Returns:
            str: Директория набора данных

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                print(settings.path_to_dataset_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                .

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_dataset_ = './dataset'
                print(settings.path_to_dataset_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                dataset

            :bdg-light:`-- 3 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_dataset_ = ''
                print(settings.path_to_dataset_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                .

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_dataset_ = [2, []]
                print(settings.path_to_dataset_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                .

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 5
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.path_to_dataset_ = 1
                print(settings.path_to_dataset_)

            .. output-cell::
                :execution-count: 5
                :linenos:

                .
        """

        return self._path_to_dataset

    @path_to_dataset_.setter
    def path_to_dataset_(self, path: str) -> None:
        """Установка директории набора данных"""

        if type(path) is str:
            self._path_to_dataset = os.path.normpath(path)

    @property
    def keys_dataset_(self):
        """Получение/установка названий ключей набора данных

        Args:
            (List[str]): Список с названиями ключей набора данных

        Returns:
            List[str]: Список с названиями ключей набора данных

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                print(settings.keys_dataset_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                [
                    'Path',
                    'Openness',
                    'Conscientiousness',
                    'Extraversion',
                    'Agreeableness',
                    'Non-Neuroticism'
                ]

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.keys_dataset_ = ['P', 'O', 'C', 'E', 'A', 'N']
                print(settings.keys_dataset_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                ['P', 'O', 'C', 'E', 'A', 'N']

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.keys_dataset_ = [{}, [], 1]
                print(settings.keys_dataset_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                [
                    'Path',
                    'Openness',
                    'Conscientiousness',
                    'Extraversion',
                    'Agreeableness',
                    'Non-Neuroticism'
                ]

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.keys_dataset_ = ['P', 'O']
                print(settings.keys_dataset_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                [
                    'Path',
                    'Openness',
                    'Conscientiousness',
                    'Extraversion',
                    'Agreeableness',
                    'Non-Neuroticism'
                ]

            :bdg-light:`-- 3 --`

            .. code-cell:: python
                :execution-count: 5
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.keys_dataset_ = []
                print(settings.keys_dataset_)

            .. output-cell::
                :execution-count: 5
                :linenos:

                [
                    'Path',
                    'Openness',
                    'Conscientiousness',
                    'Extraversion',
                    'Agreeableness',
                    'Non-Neuroticism'
                ]
        """

        return self._keys_dataset

    # Установка названий ключей набора данных
    @keys_dataset_.setter
    def keys_dataset_(self, keys: List[str]) -> None:
        """Установка названий ключей набора данных"""

        if type(keys) is list and len(keys) == len(KEYS_DATASET):
            try:
                # .capitalize()
                self._keys_dataset = [x for x in keys]
            except Exception:
                pass

        if type(keys) is list and len(keys) == len(KEYS_DATASET) - 1:
            try:
                for x in keys:
                    if type(x) is not str or not x:
                        raise TypeError
                # .capitalize()
                self._keys_dataset[1:] = [x for x in keys]
            except Exception:
                pass

    @property
    def ignore_dirs_(self) -> List[str]:
        """Получение/установка списка с директориями не входящими в выборку

        Args:
            (List[str]): Список с директориями

        Returns:
            List[str]: Список с директориями

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                print(settings.ignore_dirs_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                []

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.ignore_dirs_ = ['test', 'test_2']
                print(settings.ignore_dirs_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                ['test', 'test_2']

            :bdg-light:`-- 3 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.ignore_dirs_ = []
                print(settings.ignore_dirs_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                []

            :bdg-light:`-- 4 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.ext_ = ['1_a', '2_b']
                print(settings.ext_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                ['1_a', '2_b']

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 5
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.ignore_dirs_ = [2, []]
                print(settings.ignore_dirs_)

            .. output-cell::
                :execution-count: 5
                :linenos:

                []
        """

        return self._ignore_dirs

    @ignore_dirs_.setter
    def ignore_dirs_(self, l: List[str]) -> None:
        """Установка списка с директориями не входящими в выборку"""

        if type(l) is list:
            try:
                self._ignore_dirs = [x.lower() for x in l]
            except Exception:
                pass

    @property
    def ext_(self) -> List[str]:
        """Получение/установка расширений искомых файлов

        Args:
            (List[str]): Список с расширениями искомых файлов

        Returns:
            List[str]: Список с расширениями искомых файлов

        .. dropdown:: Примеры

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                print(settings.ext_)

            .. output-cell::
                :execution-count: 1
                :linenos:

                []

            :bdg-light:`-- 2 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.ext_ = ['.mp4']
                print(settings.ext_)

            .. output-cell::
                :execution-count: 2
                :linenos:

                ['.mp4']

            :bdg-light:`-- 3 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.ext_ = ['.mp3', '.wav']
                print(settings.ext_)

            .. output-cell::
                :execution-count: 3
                :linenos:

                ['.mp3', '.wav']

            :bdg-light:`-- 4 --`

            .. code-cell:: python
                :execution-count: 4
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.ext_ = []
                print(settings.ext_)

            .. output-cell::
                :execution-count: 4
                :linenos:

                []

            :bdg-warning:`Лучше так не делать` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 5
                :linenos:

                from oceanai.modules.core.settings import Settings

                settings = Settings()
                settings.ext_ = [2, []]
                print(settings.ext_)

            .. output-cell::
                :execution-count: 5
                :linenos:

                []
        """

        return self._ext

    @ext_.setter
    def ext_(self, ext: List[str]) -> None:
        """Установка расширений искомых файлов"""

        if type(ext) is list:
            try:
                self._ext = [x.lower() for x in ext]
            except Exception:
                pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Мультимодальное объединение информации
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################

import warnings

# Подавление Warning
for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

import os  # Взаимодействие с файловой системой
import requests  # Отправка HTTP запросов
import numpy as np  # Научные вычисления
import pandas as pd  # Обработка и анализ данных

from urllib.parse import urlparse
from urllib.error import URLError
from pathlib import Path  # Работа с путями в файловой системе
from sklearn.metrics import mean_absolute_error
from datetime import datetime  # Работа со временем

# Типы данных
from typing import List, Dict, Union, Optional, Callable

from IPython.display import clear_output

# Персональные
from oceanai.modules.lab.audio import Audio  # Аудио
from oceanai.modules.lab.video import Video  # Видео
from oceanai.modules.lab.text import Text  # Текст

import torch.nn as nn
import torch

from oceanai.modules.lab.architectures.fusion_architectures import av_model_b5, avt_model_b5


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class PredictionMessages(Audio, Video, Text):
    """Класс для сообщений

    Args:
        lang (str): Смотреть :attr:`~oceanai.modules.core.language.Language.lang`
        color_simple (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.color_simple`
        color_info (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.color_info`
        color_err (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.color_err`
        color_true (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.color_true`
        bold_text (bool): Смотреть :attr:`~oceanai.modules.core.settings.Settings.bold_text`
        num_to_df_display (int): Смотреть :attr:`~oceanai.modules.core.settings.Settings.num_to_df_display`
        text_runtime (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.text_runtime`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._av_modality: str = self._(" (мультимодальное объединение) ...")

        self._formation_av_models_b5: str = self._formation_models_b5 + self._av_modality

        self._formation_avt_model_b5: str = (
            self._("Формирование нейросетевой архитектуры модели для получения " " оценок персональных качеств")
            + self._av_modality
        )
        self._load_avt_model_weights_b5: str = (
            self._("Загрузка весов нейросетевой модели для получения " "оценок персональных качеств")
            + self._av_modality
        )

        self._model_avt_not_formation: str = (
            self._oh
            + self._(
                "нейросетевая архитектура модели для получения " "оценок по мультимодальным данным не " "сформирована"
            )
            + self._av_modality
        )

        self._load_av_models_weights_b5: str = self._load_models_weights_b5 + self._av_modality

        self._concat_av_pred_error: str = self._concat_pred_error + self._av_modality
        self._norm_av_pred_error: str = self._norm_pred_error + self._av_modality


# ######################################################################################################################
# Мультимодальное объединение
# ######################################################################################################################
@dataclass
class Prediction(PredictionMessages):
    """Класс для мультимодального объединения информации

    Args:
        lang (str): Смотреть :attr:`~oceanai.modules.core.language.Language.lang`
        color_simple (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.color_simple`
        color_info (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.color_info`
        color_err (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.color_err`
        color_true (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.color_true`
        bold_text (bool): Смотреть :attr:`~oceanai.modules.core.settings.Settings.bold_text`
        num_to_df_display (int): Смотреть :attr:`~oceanai.modules.core.settings.Settings.num_to_df_display`
        text_runtime (str): Смотреть :attr:`~oceanai.modules.core.settings.Settings.text_runtime`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        # Нейросетевые модели **nn.Module** для получения результатов оценки персональных качеств
        self._av_models_b5: Dict[str, Optional[nn.Module]] = dict(zip(self._b5["en"], [None] * len(self._b5["en"])))

        self._avt_model_b5: Optional[nn.Module] = None

        # ----------------------- Только для внутреннего использования внутри класса

        self.__len_paths: int = 0  # Количество искомых файлов
        self.__local_path: Union[Callable[[str], str], None] = None  # Локальный путь

        # Ключи для точности
        self.__df_accuracy_index: List[str] = ["MAE", "Accuracy"]
        self.__df_accuracy_index_name: str = "Metrics"
        self.__df_accuracy_mean: str = "Mean"

        clear_output(False)  # Удаление сообщения: INFO: Created TensorFlow Lite XNNPACK delegate for CPU)

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    @property
    def av_models_b5_(self) -> Dict[str, Optional[nn.Module]]:
        """Получение нейросетевых моделей **nn.Module** для получения результатов оценки персональных качеств

        Returns:
            Dict: Словарь с нейросетевыми моделями **nn.Module**

        .. dropdown:: Примеры
            :class-body: sd-pr-5

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:
                :tab-width: 8

                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                pred.load_av_models_b5(
                    show_summary = False, out = True,
                    runtime = True, run = True
                )

                pred.av_models_b5_

            .. output-cell::
                :execution-count: 1
                :linenos:

                [2022-12-08 15:21:22] Формирование нейросетевых архитектур моделей для получения результатов оценки персональных качеств (мультимодальное объединение) ...

                --- Время выполнения: 0.305 сек. ---

                {
                    'openness': <tf.keras.Model at 0x14eee5790>,
                    'conscientiousness': <tf.keras.Model at 0x14f2d9d00>,
                    'extraversion': <tf.keras.Model at 0x14f2fb190>,
                    'agreeableness': <tf.keras.Model at 0x14f2c7fd0>,
                    'non_neuroticism': <tf.keras.Model at 0x14f2ef940>
                }

            :bdg-danger:`Ошибка` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:
                :tab-width: 8

                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                pred.av_models_b5_

            .. output-cell::
                :execution-count: 2
                :linenos:
                :tab-width: 8

                {
                    'openness': None,
                    'conscientiousness': None,
                    'extraversion': None,
                    'agreeableness': None,
                    'non_neuroticism': None
                }
        """

        return self._av_models_b5

    @property
    def avt_model_b5_(self) -> Optional[nn.Module]:
        """Получение нейросетевой модели **nn.Module** для получения оценок персональных качеств

        Returns:
            Dict: Нейроаетевая модель **nn.Module**
        """

        return self._avt_model_b5

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (приватные)
    # ------------------------------------------------------------------------------------------------------------------

    def __norm_pred(self, pred_data: np.ndarray, len_nn: int = 16, out: bool = True) -> np.ndarray:
        """Нормализация оценок по экспертным и нейросетевым признакам (мультимодальная)

        .. note::
            private (приватный метод)

        Args:
            pred_data (np.ndarray): Оценки
            len_nn (int): Максимальный размер вектора оценок
            out (bool): Отображение

        Returns:
            np.ndarray: Нормализованные оценки по экспертным и нейросетевым признакам (мультимодальная)

        .. dropdown:: Примеры
            :class-body: sd-pr-5

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:
                :tab-width: 8

                import numpy as np
                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                arr = np.array([
                    [0.64113516, 0.6217892, 0.54451424, 0.6144415, 0.59334993],
                    [0.6652424, 0.63606125, 0.572305, 0.63169795, 0.612515]
                ])

                pred._Prediction__norm_pred(
                    pred_data = arr,
                    len_nn = 4,
                    out = True
                )

            .. output-cell::
                :execution-count: 1
                :linenos:
                :tab-width: 8

                array([
                    [0.64113516, 0.6217892 , 0.54451424, 0.6144415 , 0.59334993],
                    [0.6652424 , 0.63606125, 0.572305  , 0.63169795, 0.612515],
                    [0.65318878, 0.62892523, 0.55840962, 0.62306972, 0.60293247],
                    [0.65318878, 0.62892523, 0.55840962, 0.62306972, 0.60293247]
                ])

            :bdg-danger:`Ошибка` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:
                :tab-width: 8

                import numpy as np
                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                arr = np.array([])

                pred._Prediction__norm_pred(
                    pred_data = arr,
                    len_nn = 4,
                    out = True
                )

            .. output-cell::
                :execution-count: 3
                :linenos:

                [2022-10-20 22:03:17] Неверные типы или значения аргументов в "Prediction.__norm_pred" ...

                array([], dtype=float64)
        """

        try:
            # Проверка аргументов
            if (
                type(pred_data) is not np.ndarray
                or len(pred_data) < 1
                or type(len_nn) is not int
                or len_nn < 1
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.__norm_pred.__name__, out=out)
            return np.array([])
        else:
            try:
                if pred_data.shape[0] < len_nn:
                    return np.pad(pred_data, ((0, len_nn - pred_data.shape[0]), (0, 0)), "mean")
                return pred_data[:len_nn]
            except ValueError:
                self._other_error(self._norm_av_pred_error, last=False, out=out)
                return np.array([])
            except Exception:
                self._other_error(self._unknown_err, out=out)
                return np.array([])

    def __concat_pred_av(
        self,
        pred_hc_audio: np.ndarray,
        pred_nn_audio: np.ndarray,
        pred_hc_video: np.ndarray,
        pred_nn_video: np.ndarray,
        out: bool = True,
    ) -> List[Optional[np.ndarray]]:
        """Конкатенация оценок по экспертным и нейросетевым признакам (мультимодальная)

        .. note::
            private (приватный метод)

        Args:
            pred_hc_audio (np.ndarray): Оценки по экспертным признакам (аудио модальность)
            pred_nn_audio (np.ndarray): Оценки по нейросетевым признакам (аудио модальность)
            pred_hc_video (np.ndarray): Оценки по экспертным признакам (видео модальность)
            pred_nn_video (np.ndarray): Оценки по нейросетевым признакам (видео модальность)
            out (bool): Отображение

        Returns:
            List[Optional[np.ndarray]]: Конкатенированные оценки по экспертным и нейросетевым признакам

        .. dropdown:: Примеры
            :class-body: sd-pr-5

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:
                :tab-width: 8

                import numpy as np
                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                arr_hc_audio = np.array([
                    [0.64113516, 0.6217892, 0.54451424, 0.6144415, 0.59334993],
                    [0.6652424, 0.63606125, 0.572305, 0.63169795, 0.612515]
                ])

                arr_nn_audio = np.array([
                    [0.56030345, 0.7488746, 0.44648764, 0.59893465, 0.5701077],
                    [0.5900006, 0.7652722, 0.4795154, 0.6409055, 0.6088242]
                ])
                arr_hc_video = np.array([
                    [0.67113516, 0.6517892, 0.59451424, 0.6344415, 0.53334993],
                    [0.6852424, 0.62606125, 0.562305, 0.67169795, 0.672515]
                ])

                arr_nn_video = np.array([
                    [0.58030345, 0.7788746, 0.47648764, 0.53893465, 0.5901077],
                    [0.5100006, 0.7452722, 0.4495154, 0.6909055, 0.6488242]
                ])

                pred._Prediction__concat_pred_av(
                    pred_hc_audio = arr_hc_audio,
                    pred_nn_audio = arr_nn_audio,
                    pred_hc_video = arr_hc_video,
                    pred_nn_video = arr_nn_video,
                    out = True
                )
        """

        try:
            # Проверка аргументов
            if (
                type(pred_hc_audio) is not np.ndarray
                or len(pred_hc_audio) < 1
                or type(pred_nn_audio) is not np.ndarray
                or len(pred_nn_audio) < 1
                or type(pred_hc_video) is not np.ndarray
                or len(pred_hc_video) < 1
                or type(pred_nn_video) is not np.ndarray
                or len(pred_nn_video) < 1
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.__concat_pred_av.__name__, out=out)
            return []
        else:
            # Нормализация оценок по экспертным и нейросетевым признакам (аудио модальность)
            pred_hc_audio_norm = self.__norm_pred(pred_hc_audio, out=False)
            pred_nn_audio_norm = self.__norm_pred(pred_nn_audio, out=False)

            # Нормализация оценок по экспертным и нейросетевым признакам (видео модальность)
            pred_hc_video_norm = self.__norm_pred(pred_hc_video, out=False)
            pred_nn_video_norm = self.__norm_pred(pred_nn_video, out=False)

            if (
                len(pred_hc_audio_norm) == 0
                or len(pred_nn_audio_norm) == 0
                or len(pred_hc_video_norm) == 0
                or len(pred_nn_video_norm) == 0
            ):
                self._error(self._concat_av_pred_error, out=out)
                return []

            concat = []

            try:
                # Проход по всем персональным качествам личности человека
                for i in range(len(self._b5["en"])):
                    concat.append(
                        np.hstack(
                            (
                                np.asarray(pred_hc_audio_norm)[:, i],
                                np.asarray(pred_nn_audio_norm)[:, i],
                                np.asarray(pred_hc_video_norm)[:, i],
                                np.asarray(pred_nn_video_norm)[:, i],
                            )
                        )
                    )
            except IndexError:
                self._other_error(self._concat_av_pred_error, last=False, out=out)
                return []
            except Exception:
                self._other_error(self._unknown_err, out=out)
                return []

            return concat

    def __load_model_weights(
        self,
        url: str,
        force_reload: bool = True,
        info_text: str = "",
        out: bool = True,
        runtime: bool = True,
        run: bool = True,
    ) -> bool:
        """Загрузка весов нейросетевой модели

        .. note::
            private (приватный метод)

        Args:
            url (str): Полный путь к файлу с весами нейросетевой модели
            force_reload (bool): Принудительная загрузка файла с весами нейросетевой модели из сети
            info_text (str): Текст для информационного сообщения
            out (bool): Отображение
            runtime (bool): Подсчет времени выполнения
            run (bool): Блокировка выполнения

        Returns:
            bool: **True** если веса нейросетевой модели загружены, в обратном случае **False**
        """

        self._clear_notebook_history_output()  # Очистка истории вывода сообщений в ячейке Jupyter

        try:
            # Проверка аргументов
            if (
                type(url) is not str
                or not url
                or type(force_reload) is not bool
                or type(info_text) is not str
                or not info_text
                or type(out) is not bool
                or type(runtime) is not bool
                or type(run) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.__load_model_weights.__name__, out=out)
            return False
        else:
            # Блокировка выполнения
            if run is False:
                self._error(self._lock_user, out=out)
                return False

            if runtime:
                self._r_start()

            # Информационное сообщение
            self._info(info_text, last=False, out=out)

            sections = urlparse(url)  # Парсинг URL адреса

            try:
                # URL файл невалидный
                if sections.scheme == "":
                    raise requests.exceptions.InvalidURL
            except requests.exceptions.InvalidURL:
                url = os.path.normpath(url)

                try:
                    if os.path.isfile(url) is False:
                        raise FileNotFoundError  # Не файл
                except FileNotFoundError:
                    self._other_error(self._load_model_weights_error, out=out)
                    return False
                except Exception:
                    self._other_error(self._unknown_err, out=out)
                    return False
                else:
                    self._url_last_filename = url
                    return True
            else:
                try:
                    if force_reload is False:
                        clear_output(True)
                    # Загрузка файла из URL
                    res_download_file_from_url = self._download_file_from_url(
                        url=url, force_reload=force_reload, runtime=False, out=out, run=True
                    )
                except Exception:
                    self._other_error(self._unknown_err, out=out)
                    return False
                else:
                    # Файл загружен
                    if res_download_file_from_url != 200:
                        return False

                    return True
            finally:
                if runtime:
                    self._r_end(out=out)

    def __load_avt_model_b5(self, show_summary: bool = False, out: bool = True) -> Optional[nn.Module]:
        """Формирование нейросетевой архитектуры модели для получения оценок персональных качеств

        .. note::
            private (приватный метод)

        Args:
            show_summary (bool): Отображение сформированной нейросетевой архитектуры модели
            out (bool): Отображение

        Returns:
            Optional[nn.Module]:
                **None** если неверные типы или значения аргументов, в обратном случае нейросетевая модель
                **nn.Module** для получения оценок персональных качеств
        """

        try:
            # Проверка аргументов
            if type(show_summary) is not bool or type(out) is not bool:
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.__load_av_model_b5.__name__, out=out)
            return None
        else:
            model = avt_model_b5(input_shapes=[128, 128, 256, 512, 256, 2048])

            if show_summary and out:
                print(model)

            return model

    def __load_av_model_b5(self, show_summary: bool = False, out: bool = True) -> Optional[nn.Module]:
        """Формирование нейросетевой архитектуры модели для получения результата оценки персонального качества

        .. note::
            private (приватный метод)

        Args:
            show_summary (bool): Отображение сформированной нейросетевой архитектуры модели
            out (bool): Отображение

        Returns:
            Optional[nn.Module]:
                **None** если неверные типы или значения аргументов, в обратном случае нейросетевая модель
                **nn.Module** для получения результата оценки персонального качества

        .. dropdown:: Примеры
            :class-body: sd-pr-5

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:
                :tab-width: 8

                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                pred._Prediction__load_av_model_b5(
                    show_summary = True, out = True
                )

            .. output-cell::
                :execution-count: 1
                :linenos:

                Model: "model"
                _________________________________________________________________
                Layer (type)                Output Shape              Param #
                =================================================================
                input_1 (InputLayer)        [(None, 64)]              0

                dense_1 (Dense)             (None, 1)                 65

                activ_1 (Activation)        (None, 1)                 0

                =================================================================
                Total params: 65
                Trainable params: 65
                Non-trainable params: 0
                _________________________________________________________________
                <tf.keras.Model at 0x147892ee0>

            :bdg-danger:`Ошибка` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:
                :tab-width: 8

                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                pred._Prediction__load_av_model_b5(
                    show_summary = True, out = []
                )

            .. output-cell::
                :execution-count: 3
                :linenos:

                [2022-10-17 10:53:03] Неверные типы или значения аргументов в "Prediction.__load_av_model_b5" ...
        """

        try:
            # Проверка аргументов
            if type(show_summary) is not bool or type(out) is not bool:
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.__load_av_model_b5.__name__, out=out)
            return None
        else:
            model = av_model_b5()

            if show_summary and out:
                print(model)

            return model

    # ------------------------------------------------------------------------------------------------------------------
    # Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def load_av_models_b5(
        self, show_summary: bool = False, out: bool = True, runtime: bool = True, run: bool = True
    ) -> bool:
        """Формирование нейросетевых архитектур моделей для получения результатов оценки персональных качеств

        Args:
            show_summary (bool): Отображение последней сформированной нейросетевой архитектуры моделей
            out (bool): Отображение
            runtime (bool): Подсчет времени выполнения
            run (bool): Блокировка выполнения

        Returns:
            bool: **True** если нейросетевые архитектуры модели сформированы, в обратном случае **False**

        .. dropdown:: Примеры
            :class-body: sd-pr-5

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:
                :tab-width: 8

                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                pred.load_av_models_b5(
                    show_summary = True, out = True,
                    runtime = True, run = True
                )

            .. output-cell::
                :execution-count: 1
                :linenos:

                [2022-12-08 15:19:30] Формирование нейросетевых архитектур моделей для получения результатов оценки персональных качеств (мультимодальное объединение) ...

                Model: "model_4"
                _________________________________________________________________
                Layer (type)                Output Shape              Param #
                =================================================================
                input_1 (InputLayer)        [(None, 64)]              0

                dense_1 (Dense)             (None, 1)                 65

                activ_1 (Activation)        (None, 1)                 0

                =================================================================
                Total params: 65
                Trainable params: 65
                Non-trainable params: 0
                _________________________________________________________________
                --- Время выполнения: 0.141 сек. ---

                True

            :bdg-danger:`Ошибка` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 2
                :linenos:
                :tab-width: 8

                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                pred.load_av_models_b5(
                    show_summary = 1, out = True,
                    runtime = True, run = True
                )

            .. output-cell::
                :execution-count: 2
                :linenos:

                [2022-12-08 15:20:36] Неверные типы или значения аргументов в "Prediction.load_av_models_b5" ...

                False
        """

        self._clear_notebook_history_output()  # Очистка истории вывода сообщений в ячейке Jupyter

        try:
            # Проверка аргументов
            if (
                type(show_summary) is not bool
                or type(out) is not bool
                or type(runtime) is not bool
                or type(run) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.load_av_models_b5.__name__, out=out)
            return False
        else:
            # Блокировка выполнения
            if run is False:
                self._error(self._lock_user, out=out)
                return False

            if runtime:
                self._r_start()

            # Информационное сообщение
            self._info(self._formation_av_models_b5, last=False, out=False)
            if out:
                self.show_notebook_history_output()  # Отображение истории вывода сообщений в ячейке Jupyter

            for key, _ in self._av_models_b5.items():
                self._av_models_b5[key] = self.__load_av_model_b5()

            if show_summary and out:
                self._av_models_b5[key].summary()

            if runtime:
                self._r_end(out=out)

            return True

    def load_avt_model_b5(
        self, show_summary: bool = False, out: bool = True, runtime: bool = True, run: bool = True
    ) -> bool:
        """Формирование нейросетевой архитектуры модели для получения оценок персональных качеств

        Args:
            show_summary (bool): Отображение последней сформированной нейросетевой архитектуры моделей
            out (bool): Отображение
            runtime (bool): Подсчет времени выполнения
            run (bool): Блокировка выполнения

        Returns:
            bool: **True** если нейросетевая архитектура модели сформирована, в обратном случае **False**
        """

        self._clear_notebook_history_output()  # Очистка истории вывода сообщений в ячейке Jupyter

        try:
            # Проверка аргументов
            if (
                type(show_summary) is not bool
                or type(out) is not bool
                or type(runtime) is not bool
                or type(run) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.load_avt_model_b5.__name__, out=out)
            return False
        else:
            # Блокировка выполнения
            if run is False:
                self._error(self._lock_user, out=out)
                return False

            if runtime:
                self._r_start()

            # Информационное сообщение
            self._info(self._formation_avt_model_b5, last=False, out=False)
            if out:
                self.show_notebook_history_output()  # Отображение истории вывода сообщений в ячейке Jupyter

            self._avt_model_b5 = self.__load_avt_model_b5()

            if show_summary and out:
                self._avt_model_b5.summary()

            if runtime:
                self._r_end(out=out)

            return True

    def load_av_models_weights_b5(
        self,
        url_openness: str,
        url_conscientiousness: str,
        url_extraversion: str,
        url_agreeableness: str,
        url_non_neuroticism: str,
        force_reload: bool = True,
        out: bool = True,
        runtime: bool = True,
        run: bool = True,
    ) -> bool:
        """Загрузка весов нейросетевых моделей для получения результатов оценки персональных качеств

        Args:
            url_openness (str): Полный путь к файлу с весами нейросетевой модели (открытость опыту)
            url_conscientiousness (str): Полный путь к файлу с весами нейросетевой модели (добросовестность)
            url_extraversion (str): Полный путь к файлу с весами нейросетевой модели (экстраверсия)
            url_agreeableness (str): Полный путь к файлу с весами нейросетевой модели (доброжелательность)
            url_non_neuroticism (str): Полный путь к файлу с весами нейросетевой модели ('эмоциональная стабильность')
            force_reload (bool): Принудительная загрузка файлов с весами нейросетевых моделей из сети
            out (bool): Отображение
            runtime (bool): Подсчет времени выполнения
            run (bool): Блокировка выполнения

        Returns:
            bool: **True** если веса нейросетевых моделей загружены, в обратном случае **False**

        .. dropdown:: Примеры
            :class-body: sd-pr-5

            :bdg-success:`Верно` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 1
                :linenos:
                :tab-width: 8

                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                pred.load_av_models_b5(
                    show_summary = False, out = True,
                    runtime = True, run = True
                )

            .. output-cell::
                :execution-count: 1
                :linenos:

                [2022-12-08 16:56:37] Формирование нейросетевых архитектур моделей для получения результатов оценки персональных качеств (мультимодальное объединение) ...

                --- Время выполнения: 0.075 сек. ---

                True

            .. code-cell:: python
                :execution-count: 2
                :linenos:
                :tab-width: 8

                pred.path_to_save_ = './models'
                pred.chunk_size_ = 2000000

                url_openness = pred.weights_for_big5_['av']['b5']['openness']['sberdisk']
                url_conscientiousness = pred.weights_for_big5_['av']['b5']['conscientiousness']['sberdisk']
                url_extraversion = pred.weights_for_big5_['av']['b5']['extraversion']['sberdisk']
                url_agreeableness = pred.weights_for_big5_['av']['b5']['agreeableness']['sberdisk']
                url_non_neuroticism = pred.weights_for_big5_['av']['b5']['non_neuroticism']['sberdisk']

                pred.load_av_models_weights_b5(
                    url_openness = url_openness,
                    url_conscientiousness = url_conscientiousness,
                    url_extraversion = url_extraversion,
                    url_agreeableness = url_agreeableness,
                    url_non_neuroticism = url_non_neuroticism,
                    force_reload = True,
                    out = True,
                    runtime = True,
                    run = True
                )

            .. output-cell::
                :execution-count: 2
                :linenos:

                [2022-12-08 17:03:18] Загрузка весов нейросетевых моделей для получения результатов оценки персональных качеств (мультимодальное объединение) ...

                [2022-12-08 17:03:21] Загрузка файла "weights_2022-08-28_11-14-35.h5" (100.0%) ... Открытость опыту

                [2022-12-08 17:03:21] Загрузка файла "weights_2022-08-28_11-08-10.h5" (100.0%) ... Добросовестность

                [2022-12-08 17:03:21] Загрузка файла "weights_2022-08-28_11-17-57.h5" (100.0%) ... Экстраверсия

                [2022-12-08 17:03:21] Загрузка файла "weights_2022-08-28_11-25-11.h5" (100.0%) ... Доброжелательность

                [2022-12-08 17:03:21] Загрузка файла "weights_2022-06-14_21-44-09.h5" (100.0%) ... Эмоциональная стабильность

                --- Время выполнения: 3.399 сек. ---

                True

            :bdg-danger:`Ошибка` :bdg-light:`-- 1 --`

            .. code-cell:: python
                :execution-count: 3
                :linenos:
                :tab-width: 8

                from oceanai.modules.lab.prediction import Prediction

                pred = Prediction()

                pred.path_to_save_ = './models'
                pred.chunk_size_ = 2000000

                url_openness = pred.weights_for_big5_['av']['b5']['openness']['sberdisk']
                url_conscientiousness = pred.weights_for_big5_['av']['b5']['conscientiousness']['sberdisk']
                url_extraversion = pred.weights_for_big5_['av']['b5']['extraversion']['sberdisk']
                url_agreeableness = pred.weights_for_big5_['av']['b5']['agreeableness']['sberdisk']
                url_non_neuroticism = pred.weights_for_big5_['av']['b5']['non_neuroticism']['sberdisk']

                pred.load_av_models_weights_b5(
                    url_openness = url_openness,
                    url_conscientiousness = url_conscientiousness,
                    url_extraversion = url_extraversion,
                    url_agreeableness = url_agreeableness,
                    url_non_neuroticism = url_non_neuroticism,
                    force_reload = True,
                    out = True,
                    runtime = True,
                    run = True
                )

            .. output-cell::
                :execution-count: 3
                :linenos:

                [2022-12-08 17:05:32] Загрузка весов нейросетевых моделей для получения результатов оценки персональных качеств (мультимодальное объединение) ...

                [2022-12-08 17:05:32] Загрузка файла "weights_2022-08-28_11-14-35.h5" (100.0%) ...

                [2022-12-08 17:05:33] Что-то пошло не так ... не удалось загрузить веса нейросетевой модели ... Открытость опыту

                    Файл: /Users/dl/GitHub/oceanai/oceanai/modules/lab/prediction.py
                    Линия: 639
                    Метод: load_av_models_weights_b5
                    Тип ошибки: AttributeError

                [2022-12-08 17:05:33] Загрузка файла "weights_2022-08-28_11-08-10.h5" (100.0%) ...

                [2022-12-08 17:05:33] Что-то пошло не так ... не удалось загрузить веса нейросетевой модели ... Добросовестность

                    Файл: /Users/dl/GitHub/oceanai/oceanai/modules/lab/prediction.py
                    Линия: 639
                    Метод: load_av_models_weights_b5
                    Тип ошибки: AttributeError

                [2022-12-08 17:05:33] Загрузка файла "weights_2022-08-28_11-17-57.h5" (100.0%) ...

                [2022-12-08 17:05:33] Что-то пошло не так ... не удалось загрузить веса нейросетевой модели ... Экстраверсия

                    Файл: /Users/dl/GitHub/oceanai/oceanai/modules/lab/prediction.py
                    Линия: 639
                    Метод: load_av_models_weights_b5
                    Тип ошибки: AttributeError

                [2022-12-08 17:05:33] Загрузка файла "weights_2022-08-28_11-25-11.h5" (100.0%) ...

                [2022-12-08 17:05:33] Что-то пошло не так ... не удалось загрузить веса нейросетевой модели ... Доброжелательность

                    Файл: /Users/dl/GitHub/oceanai/oceanai/modules/lab/prediction.py
                    Линия: 639
                    Метод: load_av_models_weights_b5
                    Тип ошибки: AttributeError

                [2022-12-08 17:05:33] Загрузка файла "weights_2022-06-14_21-44-09.h5" (100.0%) ...

                [2022-12-08 17:05:33] Что-то пошло не так ... не удалось загрузить веса нейросетевой модели ... Эмоциональная стабильность

                    Файл: /Users/dl/GitHub/oceanai/oceanai/modules/lab/prediction.py
                    Линия: 639
                    Метод: load_av_models_weights_b5
                    Тип ошибки: AttributeError

                --- Время выполнения: 1.024 сек. ---

                False
        """

        self._clear_notebook_history_output()  # Очистка истории вывода сообщений в ячейке Jupyter

        try:
            # Проверка аргументов
            if (
                type(url_openness) is not str
                or not url_openness
                or type(url_conscientiousness) is not str
                or not url_conscientiousness
                or type(url_extraversion) is not str
                or not url_extraversion
                or type(url_agreeableness) is not str
                or not url_agreeableness
                or type(url_non_neuroticism) is not str
                or not url_non_neuroticism
                or type(force_reload) is not bool
                or type(out) is not bool
                or type(runtime) is not bool
                or type(run) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.load_av_models_weights_b5.__name__, out=out)
            return False
        else:
            # Блокировка выполнения
            if run is False:
                self._error(self._lock_user, out=out)
                return False

            if runtime:
                self._r_start()

            result_download_models = 0  # Все веса нейросетевых моделей по умолчанию загружены

            # Информационное сообщение
            self._info(self._load_av_models_weights_b5, last=False, out=out)

            # Проход по всем URL с весами нейросетевых моделей
            for cnt, url in enumerate(
                [
                    (url_openness, self._b5["ru"][0]),
                    (url_conscientiousness, self._b5["ru"][1]),
                    (url_extraversion, self._b5["ru"][2]),
                    (url_agreeableness, self._b5["ru"][3]),
                    (url_non_neuroticism, self._b5["ru"][4]),
                ]
            ):
                sections = urlparse(url[0])  # Парсинг URL адреса

                try:
                    # URL файл невалидный
                    if sections.scheme == "":
                        raise requests.exceptions.InvalidURL
                except requests.exceptions.InvalidURL:
                    url_norm = os.path.normpath(url[0])

                    try:
                        if os.path.isfile(url_norm) is False:
                            raise FileNotFoundError  # Не файл
                    except FileNotFoundError:
                        self._other_error(
                            self._load_model_weights_error + " " + self._bold_wrapper(url[1].capitalize()), out=out
                        )
                        continue
                    except Exception:
                        self._other_error(self._unknown_err, out=out)
                        continue
                    else:
                        self._url_last_filename = url_norm

                        # Отображение истории вывода сообщений в ячейке Jupyter
                        if out:
                            self.show_notebook_history_output()
                else:
                    try:
                        if force_reload is False:
                            clear_output(True)
                        # Загрузка файла из URL
                        res_download_file_from_url = self._download_file_from_url(
                            url=url[0], force_reload=force_reload, runtime=False, out=out, run=True
                        )
                    except Exception:
                        self._other_error(self._unknown_err, out=out)
                        continue
                    else:
                        # Файл загружен
                        if res_download_file_from_url != 200:
                            continue

                        try:
                            self._av_models_b5[self._b5["en"][cnt]].load_state_dict(torch.load(self._url_last_filename))
                            self._av_models_b5[self._b5["en"][cnt]].to(self._device).eval()
                        except Exception:
                            self._other_error(
                                self._load_model_weights_error + " " + self._bold_wrapper(url[1].capitalize()), out=out
                            )
                            continue
                        else:
                            self._add_last_el_notebook_history_output(self._bold_wrapper(url[1].capitalize()))

                            result_download_models += 1

            clear_output(True)
            # Отображение истории вывода сообщений в ячейке Jupyter
            if out:
                self.show_notebook_history_output()

            if runtime:
                self._r_end(out=out)

            if result_download_models != len(self._b5["ru"]):
                return False
            return True

    def get_av_union_predictions(
        self,
        depth: int = 1,
        recursive: bool = False,
        sr: int = 44100,
        window_audio: Union[int, float] = 2.0,
        step_audio: Union[int, float] = 1.0,
        reduction_fps: int = 5,
        window_video: int = 10,
        step_video: int = 5,
        lang: str = "ru",
        accuracy: bool = True,
        url_accuracy: str = "",
        logs: bool = True,
        out: bool = True,
        runtime: bool = True,
        run: bool = True,
    ) -> bool:
        """Получения прогнозов по аудио и видео (мультимодальное объединение)

        Args:
            depth (int): Глубина иерархии для получения данных
            recursive (bool): Рекурсивный поиск данных
            sr (int): Частота дискретизации
            window_audio (Union[int, float]): Размер окна сегмента аудио сигнала (в секундах)
            step_audio (Union[int, float]): Шаг сдвига окна сегмента аудио сигнала (в секундах)
            reduction_fps (int): Понижение кадровой частоты
            window_video (int): Размер окна сегмента видео сигнала (в кадрах)
            step_video (int): Шаг сдвига окна сегмента видео сигнала (в кадрах)
            lang (str): Язык
            accuracy (bool): Вычисление точности
            url_accuracy (str): Полный путь к файлу с верными предсказаниями для подсчета точности
            logs (bool): При необходимости формировать LOG файл
            out (bool): Отображение
            runtime (bool): Подсчет времени выполнения
            run (bool): Блокировка выполнения

        Returns:
            bool: **True** если прогнозы успешно получены, в обратном случае **False**

        :bdg-link-light:`Пример <../../user_guide/notebooks/Prediction-get_av_union_predictions.ipynb>`
        """

        self._clear_notebook_history_output()  # Очистка истории вывода сообщений в ячейке Jupyter

        # Сброс
        self._df_files = pd.DataFrame()  # Пустой DataFrame с данными
        self._df_accuracy = pd.DataFrame()  # Пустой DataFrame с результатами вычисления точности

        try:
            # Проверка аргументов
            if (
                type(depth) is not int
                or depth < 1
                or type(recursive) is not bool
                or type(sr) is not int
                or sr < 1
                or (
                    (type(window_audio) is not int or window_audio < 1)
                    and (type(window_audio) is not float or window_audio <= 0)
                )
                or (
                    (type(step_audio) is not int or step_audio < 1)
                    and (type(step_audio) is not float or step_audio <= 0)
                )
                or type(reduction_fps) is not int
                or reduction_fps < 1
                or type(window_video) is not int
                or window_video < 1
                or type(step_video) is not int
                or step_video < 1
                or not isinstance(lang, str)
                or lang not in self.lang_traslate
                or type(accuracy) is not bool
                or type(url_accuracy) is not str
                or type(logs) is not bool
                or type(out) is not bool
                or type(runtime) is not bool
                or type(run) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.get_av_union_predictions.__name__, out=out)
            return False
        else:
            # Блокировка выполнения
            if run is False:
                self._error(self._lock_user, out=out)
                return False

            if runtime:
                self._r_start()

            try:
                # Получение директорий, где хранятся данные
                path_to_data = self._get_paths(self.path_to_dataset_, depth, out=out)
                if type(path_to_data) is bool:
                    return False

                if type(self.keys_dataset_) is not list:
                    raise TypeError

                # Словарь для DataFrame набора данных с данными
                self._dict_of_files = dict(zip(self.keys_dataset_, [[] for _ in range(0, len(self.keys_dataset_))]))
                # Словарь для DataFrame набора данных с результатами вычисления точности
                self._dict_of_accuracy = dict(
                    zip(self.keys_dataset_[1:], [[] for _ in range(0, len(self.keys_dataset_[1:]))])
                )
            except (TypeError, FileNotFoundError):
                self._other_error(self._folder_not_found.format(self._info_wrapper(self.path_to_dataset_)), out=out)
                return False
            except Exception:
                self._other_error(self._unknown_err, out=out)
                return False
            else:
                # Вычисление точности
                if accuracy is True:
                    get_av_union_predictions_info = self._get_union_predictions_info + self._get_accuracy_info
                else:
                    get_av_union_predictions_info = self._get_union_predictions_info

                get_av_union_predictions_info += self._av_modality

                # Вычисление точности
                if accuracy is True:
                    # Информационное сообщение
                    self._info(get_av_union_predictions_info, out=out)

                    if not url_accuracy:
                        url_accuracy = self._true_traits["sberdisk"]

                    try:
                        # Загрузка верных предсказаний
                        data_true_traits = pd.read_csv(url_accuracy)
                    except (FileNotFoundError, URLError, UnicodeDecodeError):
                        self._other_error(self._load_data_true_traits_error, out=out)
                        return False
                    except Exception:
                        self._other_error(self._unknown_err, out=out)
                        return False
                    else:
                        true_traits = []
                        self._del_last_el_notebook_history_output()

                paths = []  # Пути до искомых файлов

                # Проход по всем директориям
                for curr_path in path_to_data:
                    empty = True  # По умолчанию директория пустая

                    # Рекурсивный поиск данных
                    if recursive is True:
                        g = Path(curr_path).rglob("*")
                    else:
                        g = Path(curr_path).glob("*")

                    # Формирование словаря для DataFrame
                    for p in g:
                        try:
                            if type(self.ext_) is not list or len(self.ext_) < 1:
                                raise TypeError

                            self.ext_ = [x.lower() for x in self.ext_]
                        except TypeError:
                            self._other_error(self._wrong_ext, out=out)
                            return False
                        except Exception:
                            self._other_error(self._unknown_err, out=out)
                            return False
                        else:
                            # Расширение файла соответствует расширению искомых файлов
                            if p.suffix.lower() in self.ext_:
                                if empty is True:
                                    empty = False  # Каталог не пустой

                                paths.append(p.resolve())

                try:
                    self.__len_paths = len(paths)  # Количество искомых файлов

                    if self.__len_paths == 0:
                        raise TypeError
                except TypeError:
                    self._other_error(self._files_not_found, out=out)
                    return False
                except Exception:
                    self._other_error(self._unknown_err, out=out)
                    return False
                else:
                    # Локальный путь
                    self.__local_path = lambda path: os.path.join(
                        *Path(path).parts[-abs((len(Path(path).parts) - len(Path(self.path_to_dataset_).parts))) :]
                    )

                    last = False  # Замена последнего сообщения

                    with torch.no_grad():

                        # Проход по всем искомым файлов
                        for i, curr_path in enumerate(paths):
                            if i != 0:
                                last = True

                            # Индикатор выполнения
                            self._progressbar_union_predictions(
                                get_av_union_predictions_info,
                                i,
                                self.__local_path(curr_path),
                                self.__len_paths,
                                True,
                                last,
                                out,
                            )

                            # Извлечение признаков из акустического сигнала
                            hc_audio_features, melspectrogram_audio_features = self._get_acoustic_features(
                                path=str(curr_path.resolve()),
                                sr=sr,
                                window=window_audio,
                                step=step_audio,
                                last=True,
                                out=False,
                                runtime=False,
                                run=run,
                            )

                            # Извлечение признаков из визуального сигнала
                            hc_video_features, nn_video_features, _ = self._get_visual_features(
                                path=str(curr_path.resolve()),
                                reduction_fps=reduction_fps,
                                window=window_video,
                                step=step_video,
                                lang=lang,
                                last=True,
                                out=False,
                                runtime=False,
                                run=run,
                            )

                            # Признаки из акустического сигнала извлечены
                            if (
                                type(hc_audio_features) is list
                                and type(melspectrogram_audio_features) is list
                                and type(hc_video_features) is np.ndarray
                                and type(nn_video_features) is np.ndarray
                                and len(hc_audio_features) > 0
                                and len(melspectrogram_audio_features) > 0
                                and len(hc_video_features) > 0
                                and len(nn_video_features) > 0
                            ):
                                # Коды ошибок нейросетевых моделей (аудио модальность)
                                code_error_pred_hc_audio = -1
                                code_error_pred_melspectrogram_audio = -1

                                try:
                                    # Оправка экспертных признаков в нейросетевую модель
                                    hc_audio_features = torch.from_numpy(np.array(hc_audio_features, dtype=np.float32))
                                    pred_hc_audio, _ = self.audio_model_hc_(hc_audio_features.to(self._device))
                                    pred_hc_audio = pred_hc_audio.detach().cpu()
                                except TypeError:
                                    code_error_pred_hc_audio = 1
                                except Exception:
                                    code_error_pred_melspectrogram_audio = 2

                                try:
                                    # Отправка нейросетевых признаков в нейросетевую модель
                                    melspectrogram_audio_features = torch.from_numpy(
                                        np.array(melspectrogram_audio_features, dtype=np.float32)
                                    )
                                    pred_melspectrogram_audio, _ = self.audio_model_nn_(
                                        melspectrogram_audio_features.permute(0, 3, 1, 2).to(self._device)
                                    )
                                    pred_melspectrogram_audio = pred_melspectrogram_audio.detach().cpu()
                                except TypeError:
                                    code_error_pred_melspectrogram_audio = 1
                                except Exception:
                                    code_error_pred_melspectrogram_audio = 2

                                if code_error_pred_hc_audio != -1 and code_error_pred_melspectrogram_audio != -1:
                                    self._error(self._models_audio_not_formation, out=out)
                                    return False

                                if code_error_pred_hc_audio != -1:
                                    self._error(self._model_audio_hc_not_formation, out=out)
                                    return False

                                if code_error_pred_melspectrogram_audio != -1:
                                    self._error(self._model_audio_nn_not_formation, out=out)
                                    return False

                                # Коды ошибок нейросетевых моделей (видео модальность)
                                code_error_pred_hc_video = -1
                                code_error_pred_nn_video = -1

                                try:
                                    # Оправка экспертных признаков в нейросетевую модель
                                    hc_video_features = torch.from_numpy(np.array(hc_video_features, dtype=np.float32))
                                    pred_hc_video, _ = self.video_model_hc_(hc_video_features.to(self._device))
                                    pred_hc_video = pred_hc_video.detach().cpu()
                                except TypeError:
                                    code_error_pred_hc_video = 1
                                except Exception:
                                    code_error_pred_hc_video = 2

                                try:
                                    # Отправка нейросетевых признаков в нейросетевую модель
                                    nn_video_features = torch.from_numpy(np.array(nn_video_features, dtype=np.float32))
                                    pred_nn_video, _ = self.video_model_nn_(nn_video_features.to(self._device))
                                    pred_nn_video = pred_nn_video.detach().cpu()
                                except TypeError:
                                    code_error_pred_nn_video = 1
                                except Exception:
                                    code_error_pred_nn_video = 2

                                if code_error_pred_hc_video != -1 and code_error_pred_nn_video != -1:
                                    self._error(self._models_video_not_formation, out=out)
                                    return False

                                if code_error_pred_hc_video != -1:
                                    self._error(self._model_video_hc_not_formation, out=out)
                                    return False

                                if code_error_pred_nn_video != -1:
                                    self._error(self._model_video_nn_not_formation, out=out)
                                    return False

                                # Конкатенация оценок по экспертным и нейросетевым признакам
                                union_pred = self.__concat_pred_av(
                                    pred_hc_audio.numpy(),
                                    pred_melspectrogram_audio.numpy(),
                                    pred_hc_video.numpy(),
                                    pred_nn_video.numpy(),
                                    out=out,
                                )

                                if len(union_pred) == 0:
                                    return False

                                final_pred = []

                                for cnt, (name_b5, model) in enumerate(self.av_models_b5_.items()):
                                    curr_union_pred = torch.from_numpy(np.expand_dims(union_pred[cnt], axis=0))
                                    result = model(curr_union_pred.to(self._device)).detach().cpu().numpy()[0][0]

                                    final_pred.append(result)

                                # Добавление данных в словарь для DataFrame
                                if self._append_to_list_of_files(str(curr_path.resolve()), final_pred, out) is False:
                                    return False

                                # Вычисление точности
                                if accuracy is True:
                                    try:
                                        true_trait = (
                                            data_true_traits[data_true_traits.NAME_VIDEO == curr_path.name][
                                                list(self._b5["en"])
                                            ]
                                            .values[0]
                                            .tolist()
                                        )
                                    except IndexError:
                                        self._other_error(self._expert_values_not_found, out=out)
                                        return False
                                    except Exception:
                                        self._other_error(self._unknown_err, out=out)
                                        return False
                                    else:
                                        true_traits.append(true_trait)
                            else:
                                # Добавление данных в словарь для DataFrame
                                if (
                                    self._append_to_list_of_files(
                                        str(curr_path.resolve()), [None] * len(self._b5["en"]), out
                                    )
                                    is False
                                ):
                                    return False

                                self._del_last_el_notebook_history_output()

                    # Индикатор выполнения
                    self._progressbar_union_predictions(
                        get_av_union_predictions_info,
                        self.__len_paths,
                        self.__local_path(paths[-1]),
                        self.__len_paths,
                        True,
                        last,
                        out,
                    )

                    # Отображение в DataFrame с данными
                    self._df_files = pd.DataFrame.from_dict(data=self._dict_of_files, orient="index").transpose()
                    self._df_files.index.name = self._keys_id
                    self._df_files.index += 1

                    self._df_files.index = self._df_files.index.map(str)

                    self._df_files.Path = [os.path.basename(i) for i in self._df_files.Path]

                    # Отображение
                    if out is True:
                        self._add_notebook_history_output(self._df_files.iloc[0 : self.num_to_df_display_, :])

                    # Подсчет точности
                    if accuracy is True:
                        mae_curr = []

                        for cnt, name_b5 in enumerate(self._df_files.keys().tolist()[1:]):
                            try:
                                mae_curr.append(
                                    mean_absolute_error(
                                        np.asarray(true_traits)[:, cnt], self._df_files[name_b5].to_list()
                                    )
                                )
                            except IndexError:
                                continue
                            except Exception:
                                continue

                        mae_curr = [round(float(i), 4) for i in mae_curr]
                        mae_mean = round(float(np.mean(mae_curr)), 4)
                        accuracy_curr = [round(float(i), 4) for i in 1 - np.asarray(mae_curr)]
                        accuracy_mean = round(float(np.mean(accuracy_curr)), 4)

                        for curr_acc in [mae_curr, accuracy_curr]:
                            # Добавление данных в словарь для DataFrame с результатами вычисления точности
                            if self._append_to_list_of_accuracy(curr_acc, out) is False:
                                return False

                        self._dict_of_accuracy.update({self.__df_accuracy_mean: [mae_mean, accuracy_mean]})
                        # Отображение в DataFrame с данными
                        self._df_accuracy = pd.DataFrame.from_dict(
                            data=self._dict_of_accuracy, orient="index"
                        ).transpose()
                        self._df_accuracy.index = self.__df_accuracy_index
                        self._df_accuracy.index.name = self.__df_accuracy_index_name

                        # Информационное сообщение
                        self._info(self._get_union_predictions_result, out=False)

                        # Отображение
                        if out is True:
                            self._add_notebook_history_output(self._df_accuracy.iloc[0 : self.num_to_df_display_, :])

                        self._info(
                            self._get_union_predictions_results_mean.format(
                                self._info_wrapper(str(mae_mean)), self._info_wrapper(str(accuracy_mean))
                            ),
                            out=False,
                        )

                    clear_output(True)
                    # Отображение истории вывода сообщений в ячейке Jupyter
                    if out is True:
                        self.show_notebook_history_output()

                    if logs is True:
                        # Текущее время для лог-файла
                        # см. datetime.fromtimestamp()
                        curr_ts = str(datetime.now().timestamp()).replace(".", "_")

                        name_logs_file = self.get_av_union_predictions.__name__

                        # Сохранение LOG
                        res_save_logs_df_files = self._save_logs(
                            self._df_files, name_logs_file + "_df_files_" + curr_ts
                        )

                        # Подсчет точности
                        if accuracy is True:
                            # Сохранение LOG
                            res_save_logs_df_accuracy = self._save_logs(
                                self._df_accuracy, name_logs_file + "_df_accuracy_" + curr_ts
                            )

                        if res_save_logs_df_files is True:
                            # Сохранение LOG файла/файлов
                            if accuracy is True and res_save_logs_df_accuracy is True:
                                logs_s = self._logs_saves_true
                            else:
                                logs_s = self._logs_save_true

                            self._info_true(logs_s, out=out)

                    return True
            finally:
                if runtime:
                    self._r_end(out=out)

    def load_avt_model_weights_b5(
        self,
        url: str,
        force_reload: bool = True,
        out: bool = True,
        runtime: bool = True,
        run: bool = True,
    ) -> bool:
        """Загрузка весов нейросетевой модели для получения оценок персональных качеств

        Args:
            url (str): Полный путь к файлу с весами нейросетевой модели
            force_reload (bool): Принудительная загрузка файлов с весами нейросетевых моделей из сети
            out (bool): Отображение
            runtime (bool): Подсчет времени выполнения
            run (bool): Блокировка выполнения

        Returns:
            bool: **True** если веса нейросетевой модели загружены, в обратном случае **False**
        """

        if runtime:
            self._r_start()

        if self.__load_model_weights(url, force_reload, self._load_avt_model_weights_b5, out, False, run) is True:
            try:
                self._avt_model_b5.load_state_dict(torch.load(self._url_last_filename))
                self._avt_model_b5.to(self._device).eval()
            except Exception:
                self._error(self._model_avt_not_formation, out=out)
                return False
            else:
                return True
            finally:
                if runtime:
                    self._r_end(out=out)

        return False

    def get_avt_predictions(
        self,
        depth: int = 1,
        recursive: bool = False,
        sr: int = 44100,
        window_audio: Union[int, float] = 2.0,
        step_audio: Union[int, float] = 1.0,
        reduction_fps: int = 5,
        window_video: int = 10,
        step_video: int = 5,
        asr: bool = False,
        lang: str = "ru",
        accuracy=True,
        url_accuracy: str = "",
        logs: bool = True,
        out: bool = True,
        runtime: bool = True,
        run: bool = True,
    ) -> bool:
        """Получения прогнозов по аудио, видео и тексту (мультимодальное объединение)

        Args:
            depth (int): Глубина иерархии для получения данных
            recursive (bool): Рекурсивный поиск данных
            sr (int): Частота дискретизации
            window_audio (Union[int, float]): Размер окна сегмента аудио сигнала (в секундах)
            step_audio (Union[int, float]): Шаг сдвига окна сегмента аудио сигнала (в секундах)
            reduction_fps (int): Понижение кадровой частоты
            window_video (int): Размер окна сегмента видео сигнала (в кадрах)
            step_video (int): Шаг сдвига окна сегмента видео сигнала (в кадрах)
            asr (bool): Автоматическое распознавание речи
            lang (str): Язык
            accuracy (bool): Вычисление точности
            url_accuracy (str): Полный путь к файлу с верными предсказаниями для подсчета точности
            logs (bool): При необходимости формировать LOG файл
            out (bool): Отображение
            runtime (bool): Подсчет времени выполнения
            run (bool): Блокировка выполнения

        Returns:
            bool: **True** если прогнозы успешно получены, в обратном случае **False**
        """

        self._clear_notebook_history_output()  # Очистка истории вывода сообщений в ячейке Jupyter

        # Сброс
        self._df_files = pd.DataFrame()  # Пустой DataFrame с данными
        self._df_accuracy = pd.DataFrame()  # Пустой DataFrame с результатами вычисления точности

        try:
            # Проверка аргументов
            if (
                type(depth) is not int
                or depth < 1
                or type(recursive) is not bool
                or type(sr) is not int
                or sr < 1
                or (
                    (type(window_audio) is not int or window_audio < 1)
                    and (type(window_audio) is not float or window_audio <= 0)
                )
                or (
                    (type(step_audio) is not int or step_audio < 1)
                    and (type(step_audio) is not float or step_audio <= 0)
                )
                or type(reduction_fps) is not int
                or reduction_fps < 1
                or type(window_video) is not int
                or window_video < 1
                or type(step_video) is not int
                or step_video < 1
                or type(asr) is not bool
                or not isinstance(lang, str)
                or lang not in self.lang_traslate
                or type(accuracy) is not bool
                or type(url_accuracy) is not str
                or type(logs) is not bool
                or type(out) is not bool
                or type(runtime) is not bool
                or type(run) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.get_avt_predictions.__name__, out=out)
            return False
        else:
            # Блокировка выполнения
            if run is False:
                self._error(self._lock_user, out=out)
                return False

            if runtime:
                self._r_start()

            try:
                # Получение директорий, где хранятся данные
                path_to_data = self._get_paths(self.path_to_dataset_, depth, out=out)
                if type(path_to_data) is bool:
                    return False

                if type(self.keys_dataset_) is not list:
                    raise TypeError

                # Словарь для DataFrame набора данных с данными
                self._dict_of_files = dict(zip(self.keys_dataset_, [[] for _ in range(0, len(self.keys_dataset_))]))
                # Словарь для DataFrame набора данных с результатами вычисления точности
                self._dict_of_accuracy = dict(
                    zip(self.keys_dataset_[1:], [[] for _ in range(0, len(self.keys_dataset_[1:]))])
                )
            except (TypeError, FileNotFoundError):
                self._other_error(self._folder_not_found.format(self._info_wrapper(self.path_to_dataset_)), out=out)
                return False
            except Exception:
                self._other_error(self._unknown_err, out=out)
                return False
            else:
                # Вычисление точности
                if accuracy is True:
                    get_avt_predictions_info = self._get_union_predictions_info + self._get_accuracy_info
                else:
                    get_avt_predictions_info = self._get_union_predictions_info

                get_avt_predictions_info += self._av_modality

                # Вычисление точности
                if accuracy is True:
                    # Информационное сообщение
                    self._info(get_avt_predictions_info, out=out)

                    if not url_accuracy:
                        url_accuracy = self._true_traits["sberdisk"]

                    try:
                        # Загрузка верных предсказаний
                        data_true_traits = pd.read_csv(url_accuracy)
                    except (FileNotFoundError, URLError, UnicodeDecodeError):
                        self._other_error(self._load_data_true_traits_error, out=out)
                        return False
                    except Exception:
                        self._other_error(self._unknown_err, out=out)
                        return False
                    else:
                        true_traits = []
                        self._del_last_el_notebook_history_output()

                paths = []  # Пути до искомых файлов

                # Проход по всем директориям
                for curr_path in path_to_data:
                    empty = True  # По умолчанию директория пустая

                    # Рекурсивный поиск данных
                    if recursive is True:
                        g = Path(curr_path).rglob("*")
                    else:
                        g = Path(curr_path).glob("*")

                    # Формирование словаря для DataFrame
                    for p in g:
                        try:
                            if type(self.ext_) is not list or len(self.ext_) < 1:
                                raise TypeError

                            self.ext_ = [x.lower() for x in self.ext_]
                        except TypeError:
                            self._other_error(self._wrong_ext, out=out)
                            return False
                        except Exception:
                            self._other_error(self._unknown_err, out=out)
                            return False
                        else:
                            # Расширение файла соответствует расширению искомых файлов
                            if p.suffix.lower() in self.ext_:
                                if empty is True:
                                    empty = False  # Каталог не пустой

                                paths.append(p.resolve())

                try:
                    self.__len_paths = len(paths)  # Количество искомых файлов

                    if self.__len_paths == 0:
                        raise TypeError
                except TypeError:
                    self._other_error(self._files_not_found, out=out)
                    return False
                except Exception:
                    self._other_error(self._unknown_err, out=out)
                    return False
                else:
                    # Локальный путь
                    self.__local_path = lambda path: os.path.join(
                        *Path(path).parts[-abs((len(Path(path).parts) - len(Path(self.path_to_dataset_).parts))) :]
                    )

                    last = False  # Замена последнего сообщения

                    with torch.no_grad():

                        # Проход по всем искомым файлов
                        for i, curr_path in enumerate(paths):
                            if i != 0:
                                last = True

                            # Индикатор выполнения
                            self._progressbar_union_predictions(
                                get_avt_predictions_info,
                                i,
                                self.__local_path(curr_path),
                                self.__len_paths,
                                True,
                                last,
                                out,
                            )

                            # Извлечение признаков из акустического сигнала
                            hc_audio_features, melspectrogram_audio_features = self._get_acoustic_features(
                                path=str(curr_path.resolve()),
                                sr=sr,
                                window=window_audio,
                                step=step_audio,
                                last=True,
                                out=False,
                                runtime=False,
                                run=run,
                            )

                            # Извлечение признаков из визуального сигнала
                            hc_video_features, nn_video_features, _ = self._get_visual_features(
                                path=str(curr_path.resolve()),
                                reduction_fps=reduction_fps,
                                window=window_video,
                                step=step_video,
                                lang=lang,
                                last=True,
                                out=False,
                                runtime=False,
                                run=run,
                            )

                            # Извлечение признаков из текста
                            hc_text_features, nn_text_features = self.get_text_features(
                                path=str(curr_path.resolve()),
                                asr=asr,
                                lang=lang,
                                show_text=False,
                                out=False,
                                runtime=False,
                                run=run,
                            )

                            hc_text_features = np.expand_dims(hc_text_features, axis=0)
                            nn_text_features = np.expand_dims(nn_text_features, axis=0)

                            if (
                                type(hc_audio_features) is list
                                and type(melspectrogram_audio_features) is list
                                and type(hc_video_features) is np.ndarray
                                and type(nn_video_features) is np.ndarray
                                and type(hc_text_features) is np.ndarray
                                and type(nn_text_features) is np.ndarray
                                and len(hc_audio_features) > 0
                                and len(melspectrogram_audio_features) > 0
                                and len(hc_video_features) > 0
                                and len(nn_video_features) > 0
                                and len(hc_text_features) > 0
                                and len(nn_text_features) > 0
                            ):
                                feature_lambda = lambda feature: np.concatenate(
                                    (np.mean(feature, axis=0), np.std(feature, axis=0))
                                )

                                # Коды ошибок нейросетевых моделей (аудио модальность)
                                code_error_pred_hc_audio = -1
                                code_error_pred_melspectrogram_audio = -1

                                try:
                                    # Оправка экспертных признаков в нейросетевую модель
                                    hc_audio_features = torch.from_numpy(np.array(hc_audio_features, dtype=np.float32))
                                    _, features_hc_audio = self.audio_model_hc_(hc_audio_features.to(self._device))
                                    features_hc_audio = features_hc_audio.detach().cpu()
                                except TypeError:
                                    code_error_pred_hc_audio = 1
                                except Exception:
                                    code_error_pred_melspectrogram_audio = 2

                                try:
                                    # Отправка нейросетевых признаков в нейросетевую модель
                                    melspectrogram_audio_features = torch.from_numpy(
                                        np.array(melspectrogram_audio_features, dtype=np.float32)
                                    )
                                    _, features_nn_audio = self.audio_model_nn_(
                                        melspectrogram_audio_features.permute(0, 3, 1, 2).to(self._device)
                                    )
                                    features_nn_audio = features_nn_audio.detach().cpu()
                                except TypeError:
                                    code_error_pred_melspectrogram_audio = 1
                                except Exception:
                                    code_error_pred_melspectrogram_audio = 2

                                if code_error_pred_hc_audio != -1 and code_error_pred_melspectrogram_audio != -1:
                                    self._error(self._models_audio_not_formation, out=out)
                                    return False

                                if code_error_pred_hc_audio != -1:
                                    self._error(self._model_audio_hc_not_formation, out=out)
                                    return False

                                if code_error_pred_melspectrogram_audio != -1:
                                    self._error(self._model_audio_nn_not_formation, out=out)
                                    return False

                                features_hc_audio = np.expand_dims(feature_lambda(features_hc_audio.numpy()), axis=0)
                                features_nn_audio = np.expand_dims(feature_lambda(features_nn_audio.numpy()), axis=0)

                                # Коды ошибок нейросетевых моделей (видео модальность)
                                code_error_pred_hc_video = -1
                                code_error_pred_nn_video = -1

                                try:
                                    # Оправка экспертных признаков в нейросетевую модель
                                    hc_video_features = torch.from_numpy(np.array(hc_video_features, dtype=np.float32))
                                    _, features_hc_video = self.video_model_hc_(hc_video_features.to(self._device))
                                    features_hc_video = features_hc_video.detach().cpu()
                                except TypeError:
                                    code_error_pred_hc_video = 1
                                except Exception:
                                    code_error_pred_hc_video = 2

                                try:
                                    # Отправка нейросетевых признаков в нейросетевую модель
                                    nn_video_features = torch.from_numpy(np.array(nn_video_features, dtype=np.float32))
                                    _, features_nn_video = self.video_model_nn_(nn_video_features.to(self._device))
                                    features_nn_video = features_nn_video.detach().cpu()
                                except TypeError:
                                    code_error_pred_nn_video = 1
                                except Exception:
                                    code_error_pred_nn_video = 2

                                if code_error_pred_hc_video != -1 and code_error_pred_nn_video != -1:
                                    self._error(self._models_video_not_formation, out=out)
                                    return False

                                if code_error_pred_hc_video != -1:
                                    self._error(self._model_video_hc_not_formation, out=out)
                                    return False

                                if code_error_pred_nn_video != -1:
                                    self._error(self._model_video_nn_not_formation, out=out)
                                    return False

                                features_hc_video = np.expand_dims(feature_lambda(features_hc_video.numpy()), axis=0)
                                features_nn_video = np.expand_dims(feature_lambda(features_nn_video.numpy()), axis=0)

                                # Коды ошибок нейросетевых моделей (текст)
                                code_error_pred_hc_text = -1
                                code_error_pred_nn_text = -1

                                try:
                                    # Оправка экспертных признаков в нейросетевую модель
                                    hc_text_features = torch.from_numpy(np.array(hc_text_features, dtype=np.float32))
                                    _, features_hc_text = self.text_model_hc_(hc_text_features.to(self._device))
                                    # features_hc_text = features_hc_text.detach().cpu()
                                except TypeError:
                                    code_error_pred_hc_text = 1
                                except Exception:
                                    code_error_pred_hc_text = 2

                                try:
                                    # Отправка нейросетевых признаков в нейросетевую модель
                                    nn_text_features = torch.from_numpy(np.array(nn_text_features, dtype=np.float32))
                                    _, features_nn_text = self.text_model_nn_(nn_text_features.to(self._device))
                                    # features_nn_text = features_nn_text.detach().cpu()
                                except TypeError:
                                    code_error_pred_nn_text = 1
                                except Exception:
                                    code_error_pred_nn_text = 2

                                if code_error_pred_hc_text != -1 and code_error_pred_nn_text != -1:
                                    self._error(self._model_text_not_formation, out=out)
                                    return False

                                if code_error_pred_hc_text != -1:
                                    self._error(self._model_text_hc_not_formation, out=out)
                                    return False

                                if code_error_pred_nn_text != -1:
                                    self._error(self._model_text_nn_not_formation, out=out)
                                    return False

                                try:

                                    final_pred = (
                                        self.avt_model_b5_(
                                            features_hc_text,
                                            features_nn_text,
                                            torch.from_numpy(features_hc_audio).to(self._device),
                                            torch.from_numpy(features_nn_audio).to(self._device),
                                            torch.from_numpy(features_hc_video).to(self._device),
                                            torch.from_numpy(features_nn_video).to(self._device),
                                        )
                                        .detach()
                                        .cpu()
                                        .numpy()[0]
                                        .tolist()
                                    )
                                except Exception:
                                    self._other_error(self._unknown_err, out=out)
                                    return False

                                # Добавление данных в словарь для DataFrame
                                if self._append_to_list_of_files(str(curr_path.resolve()), final_pred, out) is False:
                                    return False
                                # Вычисление точности
                                if accuracy is True:
                                    try:
                                        true_trait = (
                                            data_true_traits[data_true_traits.NAME_VIDEO == curr_path.name][
                                                list(self._b5["en"])
                                            ]
                                            .values[0]
                                            .tolist()
                                        )
                                    except IndexError:
                                        self._other_error(self._expert_values_not_found, out=out)
                                        return False
                                    except Exception:
                                        self._other_error(self._unknown_err, out=out)
                                        return False
                                    else:
                                        true_traits.append(true_trait)
                            else:
                                # Добавление данных в словарь для DataFrame
                                if (
                                    self._append_to_list_of_files(
                                        str(curr_path.resolve()), [None] * len(self._b5["en"]), out
                                    )
                                    is False
                                ):
                                    return False

                                self._del_last_el_notebook_history_output()

                    # Индикатор выполнения
                    self._progressbar_union_predictions(
                        get_avt_predictions_info,
                        self.__len_paths,
                        self.__local_path(paths[-1]),
                        self.__len_paths,
                        True,
                        last,
                        out,
                    )

                    # Отображение в DataFrame с данными
                    self._df_files = pd.DataFrame.from_dict(data=self._dict_of_files, orient="index").transpose()
                    self._df_files.index.name = self._keys_id
                    self._df_files.index += 1

                    self._df_files.index = self._df_files.index.map(str)

                    self._df_files.Path = [os.path.basename(i) for i in self._df_files.Path]
                    # Отображение
                    if out is True:
                        self._add_notebook_history_output(self._df_files.iloc[0 : self.num_to_df_display_, :])

                    # Подсчет точности
                    if accuracy is True:
                        mae_curr = []

                        for cnt, name_b5 in enumerate(self._df_files.keys().tolist()[1:]):
                            try:
                                mae_curr.append(
                                    mean_absolute_error(
                                        np.asarray(true_traits)[:, cnt], self._df_files[name_b5].to_list()
                                    )
                                )
                            except IndexError:
                                continue
                            except Exception:
                                continue

                        mae_curr = [round(float(i), 4) for i in mae_curr]
                        mae_mean = round(float(np.mean(mae_curr)), 4)
                        accuracy_curr = [round(float(i), 4) for i in 1 - np.asarray(mae_curr)]
                        accuracy_mean = round(float(np.mean(accuracy_curr)), 4)

                        for curr_acc in [mae_curr, accuracy_curr]:
                            # Добавление данных в словарь для DataFrame с результатами вычисления точности
                            if self._append_to_list_of_accuracy(curr_acc, out) is False:
                                return False

                        self._dict_of_accuracy.update({self.__df_accuracy_mean: [mae_mean, accuracy_mean]})
                        # Отображение в DataFrame с данными
                        self._df_accuracy = pd.DataFrame.from_dict(
                            data=self._dict_of_accuracy, orient="index"
                        ).transpose()
                        self._df_accuracy.index = self.__df_accuracy_index
                        self._df_accuracy.index.name = self.__df_accuracy_index_name

                        # Информационное сообщение
                        self._info(self._get_union_predictions_result, out=False)

                        # Отображение
                        if out is True:
                            self._add_notebook_history_output(self._df_accuracy.iloc[0 : self.num_to_df_display_, :])

                        self._info(
                            self._get_union_predictions_results_mean.format(
                                self._info_wrapper(str(mae_mean)), self._info_wrapper(str(accuracy_mean))
                            ),
                            out=False,
                        )

                    clear_output(True)
                    # Отображение истории вывода сообщений в ячейке Jupyter
                    if out is True:
                        self.show_notebook_history_output()

                    if logs is True:
                        # Текущее время для лог-файла
                        # см. datetime.fromtimestamp()
                        curr_ts = str(datetime.now().timestamp()).replace(".", "_")

                        name_logs_file = self.get_avt_predictions.__name__

                        # Сохранение LOG
                        res_save_logs_df_files = self._save_logs(
                            self._df_files, name_logs_file + "_df_files_" + curr_ts
                        )

                        # Подсчет точности
                        if accuracy is True:
                            # Сохранение LOG
                            res_save_logs_df_accuracy = self._save_logs(
                                self._df_accuracy, name_logs_file + "_df_accuracy_" + curr_ts
                            )

                        if res_save_logs_df_files is True:
                            # Сохранение LOG файла/файлов
                            if accuracy is True and res_save_logs_df_accuracy is True:
                                logs_s = self._logs_saves_true
                            else:
                                logs_s = self._logs_save_true

                            self._info_true(logs_s, out=out)

                    return True
            finally:
                if runtime:
                    self._r_end(out=out)

    def get_avt_predictions_gradio(
        self,
        paths: list[str] = [],
        depth: int = 1,
        recursive: bool = False,
        sr: int = 44100,
        window_audio: Union[int, float] = 2.0,
        step_audio: Union[int, float] = 1.0,
        reduction_fps: int = 5,
        window_video: int = 10,
        step_video: int = 5,
        asr: bool = False,
        lang: str = "ru",
        accuracy=True,
        url_accuracy: str = "",
        logs: bool = True,
        out: bool = True,
        runtime: bool = True,
        run: bool = True,
    ) -> bool:
        """Получения прогнозов по аудио, видео и тексту (мультимодальное объединение)

        Args:
            depth (int): Глубина иерархии для получения данных
            recursive (bool): Рекурсивный поиск данных
            sr (int): Частота дискретизации
            window_audio (Union[int, float]): Размер окна сегмента аудио сигнала (в секундах)
            step_audio (Union[int, float]): Шаг сдвига окна сегмента аудио сигнала (в секундах)
            reduction_fps (int): Понижение кадровой частоты
            window_video (int): Размер окна сегмента видео сигнала (в кадрах)
            step_video (int): Шаг сдвига окна сегмента видео сигнала (в кадрах)
            asr (bool): Автоматическое распознавание речи
            lang (str): Язык
            accuracy (bool): Вычисление точности
            url_accuracy (str): Полный путь к файлу с верными предсказаниями для подсчета точности
            logs (bool): При необходимости формировать LOG файл
            out (bool): Отображение
            runtime (bool): Подсчет времени выполнения
            run (bool): Блокировка выполнения

        Returns:
            bool: **True** если прогнозы успешно получены, в обратном случае **False**
        """

        self._clear_notebook_history_output()  # Очистка истории вывода сообщений в ячейке Jupyter

        # Сброс
        self._df_files = pd.DataFrame()  # Пустой DataFrame с данными
        self._df_accuracy = pd.DataFrame()  # Пустой DataFrame с результатами вычисления точности

        try:
            # Проверка аргументов
            if (
                type(depth) is not int
                or depth < 1
                or type(recursive) is not bool
                or type(sr) is not int
                or sr < 1
                or (
                    (type(window_audio) is not int or window_audio < 1)
                    and (type(window_audio) is not float or window_audio <= 0)
                )
                or (
                    (type(step_audio) is not int or step_audio < 1)
                    and (type(step_audio) is not float or step_audio <= 0)
                )
                or type(reduction_fps) is not int
                or reduction_fps < 1
                or type(window_video) is not int
                or window_video < 1
                or type(step_video) is not int
                or step_video < 1
                or type(asr) is not bool
                or not isinstance(lang, str)
                or lang not in self.lang_traslate
                or type(accuracy) is not bool
                or type(url_accuracy) is not str
                or type(logs) is not bool
                or type(out) is not bool
                or type(runtime) is not bool
                or type(run) is not bool
            ):
                raise TypeError
        except TypeError:
            self._inv_args(__class__.__name__, self.get_avt_predictions.__name__, out=out)
            return False
        else:
            # Блокировка выполнения
            if run is False:
                self._error(self._lock_user, out=out)
                return False

            if runtime:
                self._r_start()

            try:
                # Получение директорий, где хранятся данные
                path_to_data = self._get_paths(self.path_to_dataset_, depth, out=out)
                if type(path_to_data) is bool:
                    return False

                if type(self.keys_dataset_) is not list:
                    raise TypeError

                # Словарь для DataFrame набора данных с данными
                self._dict_of_files = dict(zip(self.keys_dataset_, [[] for _ in range(0, len(self.keys_dataset_))]))
                # Словарь для DataFrame набора данных с результатами вычисления точности
                self._dict_of_accuracy = dict(
                    zip(self.keys_dataset_[1:], [[] for _ in range(0, len(self.keys_dataset_[1:]))])
                )
            except (TypeError, FileNotFoundError):
                self._other_error(self._folder_not_found.format(self._info_wrapper(self.path_to_dataset_)), out=out)
                return False
            except Exception:
                self._other_error(self._unknown_err, out=out)
                return False
            else:
                # Вычисление точности
                if accuracy is True:
                    get_avt_predictions_info = self._get_union_predictions_info + self._get_accuracy_info
                else:
                    get_avt_predictions_info = self._get_union_predictions_info

                get_avt_predictions_info += self._av_modality

                # Вычисление точности
                if accuracy is True:
                    # Информационное сообщение
                    self._info(get_avt_predictions_info, out=out)

                    if not url_accuracy:
                        url_accuracy = self._true_traits["sberdisk"]

                    try:
                        # Загрузка верных предсказаний
                        data_true_traits = pd.read_csv(url_accuracy)
                    except (FileNotFoundError, URLError, UnicodeDecodeError):
                        self._other_error(self._load_data_true_traits_error, out=out)
                        return False
                    except Exception:
                        self._other_error(self._unknown_err, out=out)
                        return False
                    else:
                        true_traits = []
                        self._del_last_el_notebook_history_output()

                last = False  # Замена последнего сообщения

                paths = [i for i in paths if i.endswith((".mp4", ".avi", "mov", "flv"))]
                self.__len_paths = len(paths)

                with torch.no_grad():

                    # Проход по всем искомым файлов
                    for i, curr_path in enumerate(paths):
                        if i != 0:
                            last = True

                        # Извлечение признаков из акустического сигнала
                        hc_audio_features, melspectrogram_audio_features = self._get_acoustic_features(
                            path=curr_path,
                            sr=sr,
                            window=window_audio,
                            step=step_audio,
                            last=True,
                            out=True,
                            runtime=False,
                            run=run,
                        )

                        # Извлечение признаков из визуального сигнала
                        hc_video_features, nn_video_features, _ = self._get_visual_features(
                            path=curr_path,
                            reduction_fps=reduction_fps,
                            window=window_video,
                            step=step_video,
                            lang=lang,
                            last=True,
                            out=False,
                            runtime=False,
                            run=run,
                        )

                        # Извлечение признаков из текста
                        hc_text_features, nn_text_features = self.get_text_features(
                            path=curr_path,
                            asr=asr,
                            lang=lang,
                            show_text=False,
                            out=False,
                            runtime=False,
                            run=run,
                        )

                        hc_text_features = np.expand_dims(hc_text_features, axis=0)
                        nn_text_features = np.expand_dims(nn_text_features, axis=0)

                        if (
                            type(hc_audio_features) is list
                            and type(melspectrogram_audio_features) is list
                            and type(hc_video_features) is np.ndarray
                            and type(nn_video_features) is np.ndarray
                            and type(hc_text_features) is np.ndarray
                            and type(nn_text_features) is np.ndarray
                            and len(hc_audio_features) > 0
                            and len(melspectrogram_audio_features) > 0
                            and len(hc_video_features) > 0
                            and len(nn_video_features) > 0
                            and len(hc_text_features) > 0
                            and len(nn_text_features) > 0
                        ):
                            feature_lambda = lambda feature: np.concatenate(
                                (np.mean(feature, axis=0), np.std(feature, axis=0))
                            )

                            # Коды ошибок нейросетевых моделей (аудио модальность)
                            code_error_pred_hc_audio = -1
                            code_error_pred_melspectrogram_audio = -1

                            try:
                                # Оправка экспертных признаков в нейросетевую модель
                                hc_audio_features = torch.from_numpy(np.array(hc_audio_features, dtype=np.float32))
                                _, features_hc_audio = self.audio_model_hc_(hc_audio_features.to(self._device))
                                features_hc_audio = features_hc_audio.detach().cpu()
                            except TypeError:
                                code_error_pred_hc_audio = 1
                            except Exception:
                                code_error_pred_melspectrogram_audio = 2

                            try:
                                # Отправка нейросетевых признаков в нейросетевую модель
                                melspectrogram_audio_features = torch.from_numpy(
                                    np.array(melspectrogram_audio_features, dtype=np.float32)
                                )
                                _, features_nn_audio = self.audio_model_nn_(
                                    melspectrogram_audio_features.permute(0, 3, 1, 2).to(self._device)
                                )
                                features_nn_audio = features_nn_audio.detach().cpu()
                            except TypeError:
                                code_error_pred_melspectrogram_audio = 1
                            except Exception:
                                code_error_pred_melspectrogram_audio = 2

                            if code_error_pred_hc_audio != -1 and code_error_pred_melspectrogram_audio != -1:
                                self._error(self._models_audio_not_formation, out=out)
                                return False

                            if code_error_pred_hc_audio != -1:
                                self._error(self._model_audio_hc_not_formation, out=out)
                                return False

                            if code_error_pred_melspectrogram_audio != -1:
                                self._error(self._model_audio_nn_not_formation, out=out)
                                return False

                            features_hc_audio = np.expand_dims(feature_lambda(features_hc_audio.numpy()), axis=0)
                            features_nn_audio = np.expand_dims(feature_lambda(features_nn_audio.numpy()), axis=0)

                            # Коды ошибок нейросетевых моделей (видео модальность)
                            code_error_pred_hc_video = -1
                            code_error_pred_nn_video = -1

                            try:
                                # Оправка экспертных признаков в нейросетевую модель
                                hc_video_features = torch.from_numpy(np.array(hc_video_features, dtype=np.float32))
                                _, features_hc_video = self.video_model_hc_(hc_video_features.to(self._device))
                                features_hc_video = features_hc_video.detach().cpu()
                            except TypeError:
                                code_error_pred_hc_video = 1
                            except Exception:
                                code_error_pred_hc_video = 2

                            try:
                                # Отправка нейросетевых признаков в нейросетевую модель
                                nn_video_features = torch.from_numpy(np.array(nn_video_features, dtype=np.float32))
                                _, features_nn_video = self.video_model_nn_(nn_video_features.to(self._device))
                                features_nn_video = features_nn_video.detach().cpu()
                            except TypeError:
                                code_error_pred_nn_video = 1
                            except Exception:
                                code_error_pred_nn_video = 2

                            if code_error_pred_hc_video != -1 and code_error_pred_nn_video != -1:
                                self._error(self._models_video_not_formation, out=out)
                                return False

                            if code_error_pred_hc_video != -1:
                                self._error(self._model_video_hc_not_formation, out=out)
                                return False

                            if code_error_pred_nn_video != -1:
                                self._error(self._model_video_nn_not_formation, out=out)
                                return False

                            features_hc_video = np.expand_dims(feature_lambda(features_hc_video.numpy()), axis=0)
                            features_nn_video = np.expand_dims(feature_lambda(features_nn_video.numpy()), axis=0)

                            # Коды ошибок нейросетевых моделей (текст)
                            code_error_pred_hc_text = -1
                            code_error_pred_nn_text = -1

                            try:
                                # Оправка экспертных признаков в нейросетевую модель
                                hc_text_features = torch.from_numpy(np.array(hc_text_features, dtype=np.float32))
                                _, features_hc_text = self.text_model_hc_(hc_text_features.to(self._device))
                                # features_hc_text = features_hc_text.detach().cpu()
                            except TypeError:
                                code_error_pred_hc_text = 1
                            except Exception:
                                code_error_pred_hc_text = 2

                            try:
                                # Отправка нейросетевых признаков в нейросетевую модель
                                nn_text_features = torch.from_numpy(np.array(nn_text_features, dtype=np.float32))
                                _, features_nn_text = self.text_model_nn_(nn_text_features.to(self._device))
                                # features_nn_text = features_nn_text.detach().cpu()
                            except TypeError:
                                code_error_pred_nn_text = 1
                            except Exception:
                                code_error_pred_nn_text = 2

                            if code_error_pred_hc_text != -1 and code_error_pred_nn_text != -1:
                                self._error(self._model_text_not_formation, out=out)
                                return False

                            if code_error_pred_hc_text != -1:
                                self._error(self._model_text_hc_not_formation, out=out)
                                return False

                            if code_error_pred_nn_text != -1:
                                self._error(self._model_text_nn_not_formation, out=out)
                                return False

                            try:
                                final_pred = (
                                    self.avt_model_b5_(
                                        features_hc_text,
                                        features_nn_text,
                                        torch.from_numpy(features_hc_audio).to(self._device),
                                        torch.from_numpy(features_nn_audio).to(self._device),
                                        torch.from_numpy(features_hc_video).to(self._device),
                                        torch.from_numpy(features_nn_video).to(self._device),
                                    )
                                    .detach()
                                    .cpu()
                                    .numpy()[0]
                                    .tolist()
                                )
                            except Exception:
                                self._other_error(self._unknown_err, out=out)
                                return False

                            # Добавление данных в словарь для DataFrame
                            if self._append_to_list_of_files(curr_path, final_pred, out) is False:
                                return False
                            # Вычисление точности
                            if accuracy is True:
                                try:
                                    true_trait = (
                                        data_true_traits[data_true_traits.NAME_VIDEO == curr_path.name][
                                            list(self._b5["en"])
                                        ]
                                        .values[0]
                                        .tolist()
                                    )
                                except IndexError:
                                    self._other_error(self._expert_values_not_found, out=out)
                                    return False
                                except Exception:
                                    self._other_error(self._unknown_err, out=out)
                                    return False
                                else:
                                    true_traits.append(true_trait)
                        else:
                            # Добавление данных в словарь для DataFrame
                            if self._append_to_list_of_files(curr_path, [None] * len(self._b5["en"]), out) is False:
                                return False

                            self._del_last_el_notebook_history_output()

                # Отображение в DataFrame с данными
                self._df_files = pd.DataFrame.from_dict(data=self._dict_of_files, orient="index").transpose()
                self._df_files.index.name = self._keys_id
                self._df_files.index += 1

                self._df_files.index = self._df_files.index.map(str)

                self._df_files.Path = [os.path.basename(i) for i in self._df_files.Path]
                # Отображение
                if out is True:
                    self._add_notebook_history_output(self._df_files.iloc[0 : self.num_to_df_display_, :])

                # Подсчет точности
                if accuracy is True:
                    mae_curr = []

                    for cnt, name_b5 in enumerate(self._df_files.keys().tolist()[1:]):
                        try:
                            mae_curr.append(
                                mean_absolute_error(np.asarray(true_traits)[:, cnt], self._df_files[name_b5].to_list())
                            )
                        except IndexError:
                            continue
                        except Exception:
                            continue

                    mae_curr = [round(float(i), 4) for i in mae_curr]
                    mae_mean = round(float(np.mean(mae_curr)), 4)
                    accuracy_curr = [round(float(i), 4) for i in 1 - np.asarray(mae_curr)]
                    accuracy_mean = round(float(np.mean(accuracy_curr)), 4)

                    for curr_acc in [mae_curr, accuracy_curr]:
                        # Добавление данных в словарь для DataFrame с результатами вычисления точности
                        if self._append_to_list_of_accuracy(curr_acc, out) is False:
                            return False

                    self._dict_of_accuracy.update({self.__df_accuracy_mean: [mae_mean, accuracy_mean]})
                    # Отображение в DataFrame с данными
                    self._df_accuracy = pd.DataFrame.from_dict(data=self._dict_of_accuracy, orient="index").transpose()
                    self._df_accuracy.index = self.__df_accuracy_index
                    self._df_accuracy.index.name = self.__df_accuracy_index_name

                    # Информационное сообщение
                    self._info(self._get_union_predictions_result, out=False)

                    # Отображение
                    if out is True:
                        self._add_notebook_history_output(self._df_accuracy.iloc[0 : self.num_to_df_display_, :])

                    self._info(
                        self._get_union_predictions_results_mean.format(
                            self._info_wrapper(str(mae_mean)), self._info_wrapper(str(accuracy_mean))
                        ),
                        out=False,
                    )

                clear_output(True)
                # Отображение истории вывода сообщений в ячейке Jupyter
                if out is True:
                    self.show_notebook_history_output()

                if logs is True:
                    # Текущее время для лог-файла
                    # см. datetime.fromtimestamp()
                    curr_ts = str(datetime.now().timestamp()).replace(".", "_")

                    name_logs_file = self.get_avt_predictions.__name__

                    # Сохранение LOG
                    res_save_logs_df_files = self._save_logs(self._df_files, name_logs_file + "_df_files_" + curr_ts)

                    # Подсчет точности
                    if accuracy is True:
                        # Сохранение LOG
                        res_save_logs_df_accuracy = self._save_logs(
                            self._df_accuracy, name_logs_file + "_df_accuracy_" + curr_ts
                        )

                    if res_save_logs_df_files is True:
                        # Сохранение LOG файла/файлов
                        if accuracy is True and res_save_logs_df_accuracy is True:
                            logs_s = self._logs_saves_true
                        else:
                            logs_s = self._logs_save_true

                        self._info_true(logs_s, out=out)

                return True
            finally:
                if runtime:
                    self._r_end(out=out)
